@@ -28,15 +28,23 @@
 
 (deftest function-lambda-list.1
     (function-lambda-list 'cl-user::one)
-  (cl-user::a cl-user::b cl-user::c))
+  (cl-user::a cl-user::b cl-user::c)
+  nil)
+
+(deftest function-lambda-list.1a
+    (function-lambda-list 'cl-user::0-debug)
+  ()
+  t)
 
 (deftest function-lambda-list.2
     (function-lambda-list 'the)
-  (sb-c::value-type sb-c::form))
+  (sb-c::value-type sb-c::form)
+  nil)
 
 (deftest function-lambda-list.3
     (function-lambda-list #'(sb-pcl::slow-method cl-user::j (t)))
-  (sb-pcl::method-args sb-pcl::next-methods))
+  (sb-pcl::method-args sb-pcl::next-methods)
+  nil)
 
 (deftest macro-lambda-list.1
     (equal (function-lambda-list (defmacro macro-lambda-list.1-m (x b)
@@ -320,6 +328,24 @@
   (&optional sb-kernel::element-type sb-kernel::size)
   t)
 
+;;; Check correctness of METHOD-COMBINATION-LAMBDA-LIST
+(deftest method-combination-lambda-list.1
+    (method-combination-lambda-list 'standard)
+  nil)
+
+(deftest method-combination-lambda-list.2
+    (method-combination-lambda-list
+     (sb-mop:find-method-combination #'documentation 'cl-user::r nil))
+  (&optional (sb-pcl::order :most-specific-first)))
+
+(declaim (sb-ext:muffle-conditions style-warning))
+(define-method-combination long-form-mc (foo &rest args &key bar) ())
+
+(deftest method-combination-lambda-list.3
+    (method-combination-lambda-list 'long-form-mc)
+  (foo &rest args &key bar))
+
+
 ;;; Test allocation-information
 
 (defun tai (x kind info &key ignore)
@@ -384,10 +410,10 @@
 (setq sb-ext:*evaluator-mode* :compile)
 (sb-ext:defglobal *large-obj* nil)
 
-#+(and gencgc (or riscv x86 x86-64 ppc) (not win32))
+#+(and gencgc (or riscv x86 x86-64 ppc) (not win32) (not ubsan))
 (progn
   (setq *print-array* nil)
-  (setq *large-obj* (make-array (* sb-vm:gencgc-card-bytes 4)
+  (setq *large-obj* (make-array (* sb-vm:gencgc-page-bytes 4)
                                 :element-type '(unsigned-byte 8)))
   (sb-ext:gc :gen 1) ; Array won't move to a large unboxed page until GC'd
   (deftest allocation-information.5
@@ -414,30 +440,39 @@
          (= (getf props :page) page)
          (= (getf props :generation) gen)
          (eq (getf props :boxed :missing) boxedp))))
+
+(defun alloc-large-code ()
+  ;; large objets have to have to be at least 4 GC pages
+  (let ((bytes (* 4 sb-vm:gencgc-page-bytes)))
+    ;; For 32-bit in order to force allocation into a large-object page, the currently
+    ;; open code region has to be closed. Otherwise the allocation might fit in the region.
+    #-64-bit
+    (sb-alien:alien-funcall
+     (sb-alien:extern-alien "gc_close_region"
+                            (function sb-alien:void sb-alien:unsigned sb-alien:unsigned))
+     (+ (* 3 3 sb-vm:n-word-bytes) ; KLUDGE: a region is 3 words, and code_region
+                                        ; is the third in the array of regions
+        (sb-sys:find-dynamic-foreign-symbol-address "gc_alloc_region"))
+     7) ; KLUDGE: PAGE_TYPE_CODE
+    ;; A legal code object needs >= 4 boxed words. Let's use 8
+    (let ((object (sb-c:allocate-code-object nil 8 (- bytes (* 8 sb-vm:n-word-bytes)))))
+      (setq *large-obj* object)
+      ;; assert that it's large, otherwise the entire test is bogus
+      (let ((props (nth-value 1 (allocation-information object))))
+        (assert (getf props :large)))
+      object)))
+(compile 'alloc-large-code)
+
 #+gencgc
-(deftest* (allocation-information.6 :fails-on :sbcl)
+(deftest* (allocation-information.6)
     ;; Remember, all tests run after all toplevel forms have executed,
     ;; so if this were (DEFGLOBAL *LARGE-CODE* ... ) or something,
     ;; the garbage collection explicitly requested for ALLOCATION-INFORMATION.5
     ;; would have already happened, and thus affected this test as well.
     ;; So we need to make the objects within each test,
     ;; while avoiding use of lexical vars that would cause conservative pinning.
-    (multiple-value-bind (page gen)
-        (page-and-gen
-         (setq *large-obj*
-               ;; To get a large-object page, a code object has to exceed
-               ;; LARGE_OBJECT_SIZE and not fit within an open region.
-               ;; (This is a minor bug, because one should be able to
-               ;; create regions as large as desired without affecting
-               ;; determination of whether an object is large.
-               ;; Practically it means is that a small object region
-               ;; is limited to at most 3 pages)
-               ;; 32-bit machines use 64K for code allocation regions,
-               ;; but the large object size can be as small as 16K.
-               ;; 16K might fit in the free space of an open region,
-               ;; and by accident would not go on a large object page.
-               (sb-c:allocate-code-object nil 0 0
-                (max (* 4 sb-vm:gencgc-card-bytes) #-64-bit 65536))))
+    (multiple-value-bind (page gen) (page-and-gen (alloc-large-code))
+      (declare (ignorable gen))
       (declare (notinline format))
       (format (make-string-output-stream) "~%")
       (loop for i from 1 to sb-vm:+highest-normal-generation+
@@ -453,8 +488,8 @@
 (deftest allocation-information.7
     (locally
       (declare (notinline format))
-      ;; Create a bignum using 4 GC cards
-      (setq *b* (ash 1 (* sb-vm:gencgc-card-bytes sb-vm:n-byte-bits 4)))
+      ;; Create a bignum using 4 GC pages
+      (setq *b* (ash 1 (* sb-vm:gencgc-page-bytes sb-vm:n-byte-bits 4)))
       (setq *negb* (- *b*))
       (and (let ((props (get-small-bignum-allocation-information)))
              ;; *SMALL-BIGNUM* was created as a large boxed object
@@ -565,7 +600,9 @@
       (type-equal (function-type #'f)
                   (if (expect-wild-return-type-p #'f)
                       '(function (symbol) *)
-                      '(function (symbol) (values simple-string &optional)))))
+                      '(function (symbol) (values #+sb-unicode simple-string
+                                                  #-sb-unicode simple-base-string
+                                                  &optional)))))
   t)
 
 ;; Closures
@@ -761,24 +798,25 @@
 
 ;;; ASDF.  I can't even.
 (compile 'sb-introspect:map-root)
-(defun count-pointees (x simple &aux (n 0))
-  (sb-introspect:map-root (lambda (obj)
-                            (declare (ignore obj))
-                            (incf n))
-                          x
-                          :simple simple)
-  n)
+(defun list-pointees (x simple)
+  (sb-int:collect ((result))
+    (sb-introspect:map-root (lambda (obj) (result obj))
+                            x :simple simple)
+    (result)))
+(defun count-pointees (x simple)
+  (length (list-pointees x simple)))
 
 ;;; A closure points to its underlying function, all closed-over values,
 ;;; and possibly the closure's name.
-;;; #'SB-INT:CONSTANTLY-T is a nameless closure over 1 value
-(deftest map-root-closure-un (count-pointees #'SB-INT:CONSTANTLY-T nil) 2)
-;;; (SYMBOL-FUNCTION 'AND) is a named closure over 1 value
+(deftest map-root-closure-unnamed
+    (count-pointees (funcall (compile nil `(lambda (x) (lambda () x))) t) nil)
+  2)
+;;; (SYMBOL-FUNCTION 'AND) is a named closure over 1 value.
+;;; The closed-over value is AND, and the name of the closure is (:MACRO AND).
 (deftest map-root-closure-named (count-pointees (symbol-function 'and) nil) 3)
 
-;;; GFs point to their layout, implementation, slots,
-;;; and a hash-code, except that 64-bit headers can store the hash code
-;;; in the slot vector's high header bytes.
+;;; GFs point to their layout, implementation function, and slot vector.
+;;; There's also a hash-code which is stored is one of two different ways.
 ;;; However, in either case we expect only 3 referenced objects,
 ;;; because due to a strange design choice in MAP-ROOT,
 ;;; it does not invoke the funarg on fixnums (or characters).
