@@ -9,9 +9,29 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!PRETTY")
+(in-package "SB-PRETTY")
 
-#!-sb-fluid (declaim (inline index-posn posn-index posn-column))
+;;; Ancestral types
+(defstruct (queued-op (:constructor nil) (:copier nil))
+  (posn 0 :type posn))
+
+(defstruct (block-end (:include queued-op) (:copier nil))
+  (suffix nil :type (or null simple-string)))
+
+(declaim (start-block))
+(defstruct (section-start (:include queued-op)
+                          (:constructor nil)
+                          (:copier nil))
+  (depth 0 :type index)
+  (section-end nil :type (or null newline block-end)))
+
+(defstruct (newline (:include section-start) (:copier nil))
+  (kind (missing-arg)
+        :type (member :linear :fill :miser :literal :mandatory)))
+(declaim (freeze-type newline)
+         (end-block))
+
+(declaim (inline index-posn posn-index posn-column))
 (defun index-posn (index stream)
   (declare (type index index) (type pretty-stream stream)
            (values posn))
@@ -55,8 +75,8 @@
            (type (or index null) end))
   (let* ((end (or end (length string))))
     (unless (= start end)
-      (sb!impl::string-dispatch (simple-base-string
-                                 #!+sb-unicode
+      (sb-impl::string-dispatch (simple-base-string
+                                 #+sb-unicode
                                  (simple-array character (*)))
           string
         ;; For POSITION transform
@@ -101,8 +121,8 @@
                       (return))
                     (incf start count)))))))))))
 
-(defun pretty-misc (stream op &optional arg1 arg2)
-  (declare (ignore stream op arg1 arg2)))
+(defun pretty-misc (stream op arg)
+  (declare (ignore stream op arg)))
 
 ;;;; logical blocks
 
@@ -120,6 +140,7 @@
   (suffix-length 0 :type index)
   ;; The line number
   (section-start-line 0 :type index))
+(declaim (freeze-type logical-block))
 
 (defun really-start-logical-block (stream column prefix suffix)
   (let* ((blocks (pretty-stream-blocks stream))
@@ -213,7 +234,7 @@
          ,entry))))
 
 (defun enqueue-newline (stream kind)
-  (let* ((depth (length (pretty-stream-pending-blocks stream)))
+  (let* ((depth (pretty-stream-pending-blocks-length stream))
          (newline (enqueue stream newline :kind kind :depth depth)))
     (dolist (entry (pretty-stream-queue-tail stream))
       (when (and (not (eq newline entry))
@@ -227,6 +248,7 @@
                         (:copier nil))
   (kind (missing-arg) :type (member :block :current))
   (amount 0 :type fixnum))
+(declaim (freeze-type indentation))
 
 (defun enqueue-indent (stream kind amount)
   (enqueue stream indentation :kind kind :amount amount))
@@ -236,6 +258,7 @@
   (block-end nil :type (or null block-end))
   (prefix nil :type (or null simple-string))
   (suffix nil :type (or null simple-string)))
+(declaim (freeze-type block-start))
 
 (defun start-logical-block (stream prefix per-line-p suffix)
   ;; (In the PPRINT-LOGICAL-BLOCK form which calls us,
@@ -251,15 +274,15 @@
     (pretty-sout stream prefix 0 (length prefix)))
   (unless (typep suffix 'simple-string)
     (setq suffix (coerce suffix '(simple-array character (*)))))
-  (let* ((pending-blocks (pretty-stream-pending-blocks stream))
-         (start (enqueue stream block-start
-                         :prefix (and per-line-p prefix)
-                         :suffix suffix
-                         :depth (length pending-blocks))))
-    (setf (pretty-stream-pending-blocks stream)
-          (cons start pending-blocks))))
+  (let ((start (enqueue stream block-start
+                        :prefix (and per-line-p prefix)
+                        :suffix suffix
+                        :depth (pretty-stream-pending-blocks-length stream))))
+    (push start (pretty-stream-pending-blocks stream))
+    (incf (pretty-stream-pending-blocks-length stream))))
 
 (defun end-logical-block (stream)
+  (decf (pretty-stream-pending-blocks-length stream))
   (let* ((start (pop (pretty-stream-pending-blocks stream)))
          (suffix (block-start-suffix start))
          (end (enqueue stream block-end :suffix suffix)))
@@ -273,6 +296,7 @@
   (relativep nil :type (member t nil))
   (colnum 0 :type column)
   (colinc 0 :type column))
+(declaim (freeze-type queued-op)) ; and all subtypes
 
 (defun enqueue-tab (stream kind colnum colinc)
   (multiple-value-bind (sectionp relativep)
@@ -703,7 +727,6 @@ line break."
 
 ;;;; pprint-dispatch tables
 
-(define-load-time-global *standard-pprint-dispatch-table* nil)
 (define-load-time-global *initial-pprint-dispatch-table* nil)
 
 (defstruct (pprint-dispatch-entry
@@ -721,7 +744,7 @@ line break."
   ;; T iff one of the original entries.
   (initial-p (null *initial-pprint-dispatch-table*) :type boolean :read-only t)
   ;; and the associated function
-  (fun nil :type callable :read-only t))
+  (fun nil :type function-designator :read-only t))
 
 (declaim (freeze-type pprint-dispatch-entry))
 
@@ -732,6 +755,9 @@ line break."
             (pprint-dispatch-entry-priority entry)
             (pprint-dispatch-entry-initial-p entry))))
 
+(defmethod print-object ((table pprint-dispatch-table) stream)
+  (print-unreadable-object (table stream :type t :identity t)))
+
 ;; Return T iff E1 is strictly less preferable than E2.
 (defun entry< (e1 e2)
   (declare (type pprint-dispatch-entry e1 e2))
@@ -753,14 +779,14 @@ line break."
   (or (and (eq (info :type :kind type-spec) :instance)
            (let ((layout (info :type :compiler-layout type-spec)))
              (and layout
-                  (let ((info (layout-info layout)))
+                  (let ((info (wrapper-info layout)))
                     (and info
                          (let ((pred (dd-predicate-name info)))
                            (and pred (fboundp pred)
                                 (symbol-function pred))))))))
       ;; avoid compiling code for CONS, ARRAY, VECTOR, etc
-      (awhen (assoc ctype sb!c::*backend-type-predicates* :test #'type=)
-        (symbol-function (cdr it)))
+      (awhen (sb-c::backend-type-predicate ctype)
+        (symbol-function it))
       ;; OK, compile something
       (let ((name
              ;; Keep name as a string, because NAMED-LAMBDA with a symbol
@@ -780,28 +806,52 @@ line break."
 (defun copy-pprint-dispatch (&optional (table *print-pprint-dispatch*))
   (declare (type (or pprint-dispatch-table null) table))
   (let* ((orig (or table *initial-pprint-dispatch-table*))
-         (new (make-pprint-dispatch-table (copy-list (pp-dispatch-entries orig)))))
-    (replace/eql-hash-table (pp-dispatch-cons-entries new)
-                            (pp-dispatch-cons-entries orig))
+         (new (make-pprint-dispatch-table (copy-seq (pp-dispatch-entries orig))
+                                          (pp-dispatch-number-matchable-p orig)
+                                          (pp-dispatch-only-initial-entries orig))))
+    (hash-table-replace (pp-dispatch-cons-entries new) (pp-dispatch-cons-entries orig))
     new))
 
 (defun pprint-dispatch (object &optional (table *print-pprint-dispatch*))
   (declare (type (or pprint-dispatch-table null) table))
   (let* ((table (or table *initial-pprint-dispatch-table*))
+         (possibly-matchable
+          (if (pp-dispatch-only-initial-entries table)
+              ;; Filters out SYMBOL, INSTANCE, FUNCTION, NUMBER, most arrays,
+              ;; and many other types very quickly. As horrible as the code emitted
+              ;; for this check is, it's still 3x to 4x faster than >= 6 funcalls.
+              ;; The inefficiency of the array test can be fixed by transforming
+              ;; it to something clever like
+              ;;  (and (> widetag #x80) (logbitp (ash widetag -2) #xNNNN)) ; some bit pattern
+              ;; at the source level, or in the backend.
+              ;; See !PPRINT-COLD-INIT for the possibly matchable types.
+              (typep object '(or cons
+                                 (and array (not (or string bit-vector)))
+                                 sb-impl::comma))
+              ;; We could do something similar to the POSSIBLY-MATCHABLE test
+              ;; based on tracking separately whether the user added any non-cons
+              ;; entries and cons entries. So if no non-cons entries were added,
+              ;; then the table can't match OBJECT if it is an atom and in the
+              ;; set of atoms matched by the preceding TYPEP test.
+              (or (not (numberp object))
+                  (pp-dispatch-number-matchable-p table))))
          (entry
-          (when (or (not (numberp object)) (pp-dispatch-number-matchable-p table))
+          (when possibly-matchable
             (let ((cons-entry
                    (and (consp object)
-                        (gethash (car object) (pp-dispatch-cons-entries table)))))
-              (if (not cons-entry)
-                  (dolist (entry (pp-dispatch-entries table) nil)
-                    (when (funcall (pprint-dispatch-entry-test-fn entry) object)
-                      (return entry)))
-                  (dolist (entry (pp-dispatch-entries table) cons-entry)
-                    (when (entry< entry cons-entry)
-                      (return cons-entry))
-                    (when (funcall (pprint-dispatch-entry-test-fn entry) object)
-                      (return entry))))))))
+                        (sb-impl::gethash/eql (car object) (pp-dispatch-cons-entries table) nil))))
+              (cond ((not cons-entry)
+                     (dovector (entry (pp-dispatch-entries table) nil)
+                       (when (funcall (pprint-dispatch-entry-test-fn entry) object)
+                         (return entry))))
+                    ((pp-dispatch-only-initial-entries table)
+                     cons-entry)
+                    (t
+                     (dovector (entry (pp-dispatch-entries table) cons-entry)
+                       (when (entry< entry cons-entry)
+                         (return cons-entry))
+                       (when (funcall (pprint-dispatch-entry-test-fn entry) object)
+                         (return entry)))))))))
     (if entry
         (values (pprint-dispatch-entry-fun entry) t)
         (values #'output-ugly-object nil))))
@@ -812,9 +862,9 @@ line break."
             :operation operation)))
 
 (defun defer-type-checker (entry)
-  (let ((saved-nonce sb!c::*type-cache-nonce*))
+  (let ((saved-nonce sb-kernel::*type-cache-nonce*))
     (lambda (obj)
-      (let ((nonce sb!c::*type-cache-nonce*))
+      (let ((nonce sb-kernel::*type-cache-nonce*))
         (if (eq nonce saved-nonce)
             nil
             (let ((ctype (specifier-type (pprint-dispatch-entry-type entry))))
@@ -837,18 +887,17 @@ line break."
 ;; (cons (eql foo) cons) and (cons (eql foo) bit-vector) as two FOO entries.
 (defun set-pprint-dispatch (type function &optional
                             (priority 0) (table *print-pprint-dispatch*))
-  (declare (type (or null callable) function)
+  (declare (type function-designator function)
            (type real priority)
            (type pprint-dispatch-table table))
   (declare (explicit-check))
-  #!+(and sb-show (host-feature sb-xc)) (format t "* SET-PP-DISPATCH ~S~%" type)
   (assert-not-standard-pprint-dispatch-table table 'set-pprint-dispatch)
   (let* ((ctype (or (handler-bind
                         ((parse-unknown-type
                           (lambda (c)
                             (warn "~S is not a recognized type specifier"
                                   (parse-unknown-type-specifier c)))))
-                      (sb!c::careful-specifier-type type))
+                      (sb-c::careful-specifier-type type))
                     (error "~S is not a valid type-specifier" type)))
          (consp (and (cons-type-p ctype)
                      (eq (cons-type-cdr-type ctype) *universal-type*)
@@ -870,40 +919,39 @@ line break."
             (if function
                 (setf (gethash key hashtable) entry)
                 (remhash key hashtable))))
-        (setf (pp-dispatch-entries table)
-              (let ((list (delete type (pp-dispatch-entries table)
-                                  :key #'pprint-dispatch-entry-type
-                                  :test #'equal)))
+        (setf (pp-dispatch-only-initial-entries table) nil
+              (pp-dispatch-entries table)
+              (let ((old (remove type (pp-dispatch-entries table)
+                                 :key #'pprint-dispatch-entry-type
+                                 :test #'equal)))
                 (if function
                     ;; ENTRY< is T if lower in priority, which should sort to
                     ;; the end, but MERGE's predicate wants T for the (a,b) pair
                     ;; if 'a' should go in front of 'b', so swap them.
                     ;; (COMPLEMENT #'entry<) is unstable wrt insertion order.
-                    (merge 'list list (list entry) (lambda (a b) (entry< b a)))
-                    list)))))
+                    (merge 'vector old (list entry) (lambda (a b) (entry< b a)))
+                    old)))))
   nil)
 
 ;;;; standard pretty-printing routines
 
 (defun pprint-array (stream array)
-  (cond ((and (null *print-array*) (null *print-readably*))
-         (output-ugly-object stream array))
+  (cond ((or (null (array-element-type array))
+             (and (null *print-array*) (null *print-readably*)))
+         (print-object array stream))
         ((and *print-readably*
               (not (array-readably-printable-p array)))
-         (sb!impl::output-unreadable-array-readably array stream))
+         (sb-impl::output-unreadable-array-readably array stream))
         ((vectorp array)
-         (pprint-vector stream array))
+         (pprint-logical-block (stream nil :prefix "#(" :suffix ")")
+           (dotimes (i (length array))
+             (unless (zerop i)
+               (format stream " ~:_"))
+             (pprint-pop)
+             (output-object (aref array i) stream))))
         (t
          (pprint-multi-dim-array stream array))))
 
-(defun pprint-vector (stream vector)
-  (pprint-logical-block (stream nil :prefix "#(" :suffix ")")
-    (dotimes (i (length vector))
-      (unless (zerop i)
-        (format stream " ~:_"))
-      (pprint-pop)
-      (output-object (aref vector i) stream))))
-
 (defun pprint-multi-dim-array (stream array)
   (funcall (formatter "#~DA") stream (array-rank array))
   (with-array-data ((data array) (start) (end))
@@ -993,17 +1041,21 @@ line break."
 (defun pprint-lambda (stream list &rest noise)
   (declare (ignore noise))
   (funcall (formatter
+            ;; META: I disagree with the claim that we are not ANSI-compliant.
+            ;; The time at which INTERN occurs does not seem to me to lie within
+            ;; the scope of what is meant by "equivalent" with respect to behavior
+            ;; of the format control.
             ;; KLUDGE: This format string, and other format strings which also
-            ;; refer to SB!PRETTY, rely on the current SBCL not-quite-ANSI
+            ;; refer to SB-PRETTY, rely on the current SBCL not-quite-ANSI
             ;; behavior of FORMATTER in order to make code which survives the
-            ;; transition when SB!PRETTY is renamed to SB-PRETTY after cold
-            ;; init. (ANSI says that the FORMATTER functions should be
+            ;; transition when SB-PRETTY is renamed.
+            ;; (ANSI says that the FORMATTER functions should be
             ;; equivalent to the format string, but the SBCL FORMATTER
             ;; functions contain references to package objects, not package
             ;; names, so they keep right on going if the packages are renamed.)
             ;; If our FORMATTER behavior is ever made more compliant, the code
             ;; here will have to change. -- WHN 19991207
-            "~:<~^~W~^~3I ~:_~/SB!PRETTY:PPRINT-LAMBDA-LIST/~1I~@{ ~_~W~}~:>")
+            "~:<~^~W~^~3I ~:_~/SB-PRETTY:PPRINT-LAMBDA-LIST/~1I~@{ ~_~W~}~:>")
            stream
            list))
 
@@ -1020,7 +1072,7 @@ line break."
            (and (consp (cddr list))
                 (not (eq :in (third list)))))
       (funcall (formatter
-                "~:<~^~W~^ ~@_~:<~@{~:<~^~W~^~3I ~:_~/SB!PRETTY:PPRINT-LAMBDA-LIST/~1I~:@_~@{~W~^ ~_~}~:>~^ ~_~}~:>~1I~@:_~@{~W~^ ~_~}~:>")
+                "~:<~^~W~^ ~@_~:<~@{~:<~^~W~^~3I ~:_~/SB-PRETTY:PPRINT-LAMBDA-LIST/~1I~:@_~@{~W~^ ~_~}~:>~^ ~_~}~:>~1I~@:_~@{~W~^ ~_~}~:>")
                stream
                list)
       ;; for printing function names like (flet foo)
@@ -1034,7 +1086,7 @@ line break."
 
 (defun pprint-let (stream list &rest noise)
   (declare (ignore noise))
-  (funcall (formatter "~:<~^~W~^ ~@_~:<~@{~:<~^~W~@{ ~_~W~}~:>~^ ~_~}~:>~1I~:@_~@{~W~^ ~_~}~:>")
+  (funcall (formatter "~:<~^~W~^ ~@_~:<~@{~:<~^~W~@{ ~_~W~}~:>~^ ~_~}~:>~1I~^~:@_~@{~W~^ ~_~}~:>")
            stream
            list))
 
@@ -1140,14 +1192,14 @@ line break."
 (defun pprint-case (stream list &rest noise)
   (declare (ignore noise))
   (funcall (formatter
-            "~:<~^~W~^ ~3I~:_~W~1I~@{ ~_~:<~^~:/SB!PRETTY:PPRINT-FILL/~^~@{ ~_~W~}~:>~}~:>")
+            "~:<~^~W~^ ~3I~:_~W~1I~@{ ~_~:<~^~:/SB-PRETTY:PPRINT-FILL/~^~@{ ~_~W~}~:>~}~:>")
            stream
            list))
 
 (defun pprint-defun (stream list &rest noise)
   (declare (ignore noise))
   (funcall (formatter
-            "~:<~^~W~^ ~@_~:I~W~^ ~:_~/SB!PRETTY:PPRINT-LAMBDA-LIST/~1I~@{ ~_~W~}~:>")
+            "~:<~^~W~^ ~@_~:I~W~^ ~:_~/SB-PRETTY:PPRINT-LAMBDA-LIST/~1I~@{ ~_~W~}~:>")
            stream
            list))
 
@@ -1158,7 +1210,7 @@ line break."
            (consp (third list)))
       (pprint-defun stream list)
       (funcall (formatter
-                "~:<~^~W~^ ~@_~:I~W~^ ~W~^ ~:_~/SB!PRETTY:PPRINT-LAMBDA-LIST/~1I~@{ ~_~W~}~:>")
+                "~:<~^~W~^ ~@_~:I~W~^ ~W~^ ~:_~/SB-PRETTY:PPRINT-LAMBDA-LIST/~1I~@{ ~_~W~}~:>")
                stream
                list)))
 
@@ -1172,7 +1224,7 @@ line break."
 (defun pprint-destructuring-bind (stream list &rest noise)
   (declare (ignore noise))
   (funcall (formatter
-            "~:<~^~W~^~3I ~_~:/SB!PRETTY:PPRINT-LAMBDA-LIST/~^ ~_~W~^~1I~@{ ~_~W~}~:>")
+            "~:<~^~W~^~3I ~_~:/SB-PRETTY:PPRINT-LAMBDA-LIST/~^ ~_~W~^~1I~@{ ~_~W~}~:>")
            stream list))
 
 (defun pprint-do (stream list &rest noise)
@@ -1346,7 +1398,7 @@ line break."
 (defmacro with-pretty-stream ((stream-var
                                      &optional (stream-expression stream-var))
                                     &body body)
-  (let ((flet-name (sb!xc:gensym "WITH-PRETTY-STREAM")))
+  (let ((flet-name (gensym "WITH-PRETTY-STREAM")))
     `(flet ((,flet-name (,stream-var)
               ,@body))
        (let ((stream ,stream-expression))
@@ -1433,32 +1485,63 @@ line break."
           (t
            (incf (car state))))))
 
+;;; Warm bootup is slightly less brittle if we can avoid first having to run
+;;; the compiler to make some predicates for the initial PPD type specifiers.
+;;; In particular I'm trying to eradicate a bunch of PARSE-UNKNOWN conditions
+;;; that occur, e.g. installing (AND ARRAY (NOT (OR STRING BIT-VECTOR)))
+;;; entails (compile nil '(typep sb-pretty::object <that-type>))
+;;; -> (sb-c::find-free-fun typep)
+;;; -> (sb-kernel:specifier-type (function (t (or cons symbol sb-kernel:classoid class) ...)))
+;;; -> (sb-kernel::parse-args-types ...)
+;;; -> (sb-kernel::%parse-type CLASS)
+;;; where the type specifier for CLASS is as yet unknown
+(defmacro initial-entry (specifier handler priority &optional predicate)
+  `(make-pprint-dispatch-entry
+    ',specifier ,priority #',handler
+    ,(or predicate
+         `(named-lambda ,(format nil "~A-P" handler) (x)
+            ,(sb-c::source-transform-typep 'x specifier)))))
+
+
+;;;; Interface seen by regular (ugly) printer.
+
+;;; OUTPUT-PRETTY-OBJECT is called by OUTPUT-OBJECT when
+;;; *PRINT-PRETTY* is true.
+(defun output-pretty-object (stream fun object)
+  (with-pretty-stream (stream)
+    (funcall fun stream object)))
+
 (defun !pprint-cold-init ()
   (/show0 "entering !PPRINT-COLD-INIT")
   ;; Kludge: We set *STANDARD-PP-D-TABLE* to a new table even though
   ;; it's going to be set to a copy of *INITIAL-PP-D-T* below because
   ;; it's used in WITH-STANDARD-IO-SYNTAX, and condition reportery
   ;; possibly performed in the following extent may use W-S-IO-SYNTAX.
-  (setf *standard-pprint-dispatch-table* (make-pprint-dispatch-table))
+  (setf *standard-pprint-dispatch-table* (make-pprint-dispatch-table #() nil nil))
   (setf *initial-pprint-dispatch-table* nil)
-  (let ((*print-pprint-dispatch* (make-pprint-dispatch-table)))
-    (/show0 "doing SET-PPRINT-DISPATCH for regular types")
-    (set-pprint-dispatch '(and array (not (or string bit-vector))) 'pprint-array)
-    ;; MACRO-FUNCTION must have effectively higher priority than FBOUNDP.
-    ;; The implementation happens to check identical priorities in the order added,
-    ;; but that's unspecified behavior.  Both must be _strictly_ lower than the
-    ;; default cons entries though.
-    (set-pprint-dispatch '(cons (and symbol (satisfies macro-function)))
-                         'pprint-macro-call -1)
-    (set-pprint-dispatch '(cons (and symbol (satisfies fboundp)))
-                         'pprint-fun-call -1)
-    (set-pprint-dispatch '(cons symbol)
-                         'pprint-data-list -2)
-    (set-pprint-dispatch 'cons 'pprint-fill -2)
-    (set-pprint-dispatch 'sb!impl::comma 'pprint-unquoting-comma -3)
+  (let* ((initial-entries
+          (vector
+           ;; * PLEASE NOTE : If you change these definitions, then you may need to adjust
+           ;; the computation of POSSIBLY-MATCHABLE in PPRINT-DISPATCH.
+           ;;
+           ;; Assign a lower priority than for the cons entries below, making
+           ;; fewer type tests when dispatching.
+           (initial-entry (and array (not (or string bit-vector)))
+                          pprint-array -1)
+           ;; MACRO-FUNCTION must have effectively higher priority than FBOUNDP.
+           ;; The implementation happens to check identical priorities in the order added,
+           ;; but that's unspecified behavior.  Both must be _strictly_ lower than the
+           ;; default cons entries though.
+           (initial-entry (cons (and symbol (satisfies macro-function)))
+                          pprint-macro-call -1)
+           (initial-entry (cons (and symbol (satisfies fboundp))) pprint-fun-call -1)
+           (initial-entry (cons symbol) pprint-data-list -2)
+           (initial-entry cons pprint-fill -2 #'consp)
+           (initial-entry sb-impl::comma pprint-unquoting-comma -3 #'comma-p)))
+         (*print-pprint-dispatch*
+          (make-pprint-dispatch-table initial-entries nil nil)))
     ;; cons cells with interesting things for the car
     (/show0 "doing SET-PPRINT-DISPATCH for CONS with interesting CAR")
-
     (dolist (magic-form '((lambda pprint-lambda)
                           ((declare declaim) pprint-declare)
 
@@ -1510,7 +1593,8 @@ line break."
       ;; The sharing of dispatch entries is inconsequential.
       (set-pprint-dispatch `(cons (member ,@(ensure-list (first magic-form))))
                            (second magic-form)))
-    (setf *initial-pprint-dispatch-table* *print-pprint-dispatch*))
+    (setf (pp-dispatch-only-initial-entries *print-pprint-dispatch*) t
+          *initial-pprint-dispatch-table* *print-pprint-dispatch*))
 
   (setf *standard-pprint-dispatch-table*
         (copy-pprint-dispatch *initial-pprint-dispatch-table*))
