@@ -934,7 +934,7 @@
   (:arg-types unsigned-num unsigned-num)
   (:note "inline (unsigned-byte 64) comparison"))
 
-(defmacro define-conditional-vop (tran signed unsigned)
+(defmacro define-conditional-vop (tran signed unsigned &optional addend addend-signed addend-unsigned)
   `(progn
      ,@(loop for (suffix cost signed-p) in
              '((/fixnum 4 t)
@@ -954,14 +954,31 @@
        (:arg-types (:or tagged-num signed-num unsigned-num)
                    (:constant (or signed-word word)))
        (:info y)
+       (:vop-var vop)
        (:policy :fast-safe)
        ,(if (eq signed unsigned)
             `(:conditional ,signed)
             `(:conditional
               :after-sc-selection
-              (if (sc-is x unsigned-reg)
-                  ,unsigned
-                  ,signed)))
+              (flet ((try (y)
+                       (let ((y (if (sc-is x any-reg)
+                                    (fixnumize y)
+                                    y)))
+                         (flet ((try (constant)
+                                  (add-sub-immediate-p constant)))
+                           (or (try y)
+                               (try (ldb (byte 64 0) (- y))))))))
+                (cond ((and (not (try y))
+                            (try (+ y ,addend)))
+                       (setf (car (vop-codegen-info vop))
+                             (+ y ,addend))
+                       (if (sc-is x unsigned-reg)
+                           ,addend-unsigned
+                           ,addend-signed))
+                      (t
+                       (if (sc-is x unsigned-reg)
+                           ,unsigned
+                           ,signed))))))
        (:generator 2
          (let ((y (if (sc-is x any-reg)
                       (fixnumize y)
@@ -976,8 +993,8 @@
                  (try (ldb (byte 64 0) (- y)) t)
                  (inst cmp x (load-immediate-word tmp-tn y)))))))))
 
-(define-conditional-vop < :lt :lo)
-(define-conditional-vop > :gt :hi)
+(define-conditional-vop < :lt :lo -1 :le :ls)
+(define-conditional-vop > :gt :hi 1 :ge :hs)
 (define-conditional-vop eql :eq :eq)
 
 (define-vop (<-unsigned-signed)
@@ -1067,12 +1084,27 @@
   (:policy :fast-safe)
   (:conditional :ne)
   (:generator 2
-    (let ((y (if (sc-is x any-reg)
-                 (fixnumize y)
-                 y)))
-      (if (encode-logical-immediate y)
-          (inst tst x y)
-          (inst tst x (load-immediate-word tmp-tn y))))))
+    (block nil
+      (if (= y most-positive-word)
+          (inst cmp x 0)
+          (let ((y (if (sc-is x any-reg)
+                       (cond ((fixnump y)
+                              (fixnumize y))
+                             ((let ((y (ldb (byte 64 0) y)))
+                                ;; Only a negative fixnum will match that bit,
+                                ;; so if the next bit is also 1 it can be shifted left.
+                                (when (and (= (ldb (byte 1 62) y) 1)
+                                           (encode-logical-immediate
+                                            (ldb (byte 64 0) (ash y 1))))
+                                  (return (inst tst x (ldb (byte 64 0) (ash y 1)))))))
+                             (t
+                              (return
+                                (inst tst (load-immediate-word tmp-tn y)
+                                      (asr x n-fixnum-tag-bits)))))
+                       y)))
+            (if (encode-logical-immediate y)
+                (inst tst x y)
+                (inst tst x (load-immediate-word tmp-tn y))))))))
 
 (define-source-transform lognand (x y)
   `(lognot (logand ,x ,y)))
@@ -1179,6 +1211,17 @@
 (define-full-setter bignum-set * bignum-digits-offset other-pointer-lowtag
   (unsigned-reg) unsigned-num sb-bignum:%bignum-set)
 
+(define-partial-reffer sb-c::%half-bignum-ref bignum
+  :word nil bignum-digits-offset other-pointer-lowtag (unsigned-reg signed-reg)
+  positive-fixnum
+  sb-bignum:%half-bignum-ref)
+
+(define-partial-setter sb-c::%half-bignum-set bignum
+  :word bignum-digits-offset other-pointer-lowtag (unsigned-reg signed-reg)
+  positive-fixnum
+  (setf sb-bignum:%half-bignum-ref))
+
+
 (define-vop (digit-0-or-plus)
   (:translate sb-bignum:%digit-0-or-plusp)
   (:policy :fast-safe)
@@ -1187,8 +1230,8 @@
   (:conditional)
   (:info target not-p)
   (:generator 2
-     (inst cmp digit 0)
-     (inst b (if not-p :lt :ge) target)))
+    (inst cmp digit 0)
+    (inst b (if not-p :lt :ge) target)))
 
 (define-vop (add-w/carry)
   (:translate sb-bignum:%add-with-carry)
@@ -1222,7 +1265,7 @@
     (inst cmp c 1) ;; Set carry if (fixnum 0 or 1) c=0, else clear.
     (inst sbcs result a b)
     (unless (eq (tn-kind borrow) :unused)
-     (inst cset borrow :cs))))
+      (inst cset borrow :cs))))
 
 (define-vop (bignum-mult-and-add-3-arg)
   (:translate sb-bignum:%multiply-and-add)
@@ -1333,6 +1376,18 @@
         (unless (= i 64)
           (inst adc rem rem rem))))))
 
+(define-vop (half-bignum-floor bignum-floor)
+  (:translate sb-bignum:%half-bigfloor)
+  (:args (div-high :scs (unsigned-reg) :to :save)
+         (div-low :scs (unsigned-reg) :target x)
+         (divisor :scs (unsigned-reg) :to (:result 1)))
+  (:temporary (:sc unsigned-reg :from (:argument 1)) x)
+  (:generator 30
+    (move x div-low)
+    (inst bfm x div-high 32 31)
+    (inst udiv quo x divisor)
+    (inst msub rem quo divisor x)))
+
 (define-vop (signify-digit)
   (:translate sb-bignum:%fixnum-digit-with-correct-sign)
   (:policy :fast-safe)
