@@ -97,17 +97,8 @@
     other-pointer-lowtag))
 
 (defconstant-eqx fixnum-lowtags
-    #.(let ((fixtags nil))
-        (do-external-symbols (sym "SB-VM")
-          (let* ((name (symbol-name sym))
-                 (len (length name)))
-            (when (and (boundp sym)
-                       (integerp (symbol-value sym))
-                       (> len 7)
-                       (string= name "-LOWTAG" :start1 (- len 7))
-                       (zerop (logand (symbol-value sym) fixnum-tag-mask)))
-              (push sym fixtags))))
-        `',(sort fixtags #'string< :key #'symbol-name))
+    '#.(loop for i from 0 to lowtag-mask
+             when (zerop (logand i fixnum-tag-mask)) collect i)
   #'equal)
 
 ;;; the heap types, stored in 8 bits of the header of an object on the
@@ -180,122 +171,177 @@
           ;; a long, horrible death.  --njf, 2004-08-09
           :start #.(+ (ash 1 n-lowtag-bits) other-immediate-0-lowtag)
           :step 4)
-                                            ; +unicode -unicode
-;;                             Word bits    ;  32 | 64  32 | 64
-                                            ;------------------
-                                            ; [ all numbers are hex ]
-  bignum-widetag                            ;  0A   11  0A   11       \
-  ratio-widetag                             ;  0E   15  0E   15        |
-  single-float-widetag                      ;  12   19  12   19        |
-  double-float-widetag                      ;  16   1D  16   1D        | EQL-hash picks off this
-  complex-widetag                           ;  1A   21  1A   21        | range of widetags
-  complex-single-float-widetag              ;  1E   25  1E   25        |
-  complex-double-float-widetag              ;  22   29  22   29        |
-                                            ;                          |
-  symbol-widetag                            ;  26   2D  26   2D       /
+;;                             Word bits          ;  32 | 64
+                                                  ;---------
+                                                  ; [ all numbers are hex ]
 
-  #-64-bit instance-widetag                 ;  2A       2A
-  #-64-bit funcallable-instance-widetag     ;  2E       2E
-  #-64-bit simple-fun-widetag               ;  32       32
-  #-64-bit closure-widetag                  ;  36       36
-  #-64-bit code-header-widetag              ;  3A       3A
+  ;; NOTE: If changing the widetags bracketed by the comment about EQL-hash,
+  ;; check that the definition of stable_eql_hash_p() in gc-common is correct.
+  ;; FIXME: EQL-HASH should treat SAP like a number
+  ;; -- start of numeric widetags --
+  bignum-widetag                                  ;  0A   11       \
+  ratio-widetag                                   ;  0E   15       |
+  single-float-widetag                            ;  12   19       |
+  double-float-widetag                            ;  16   1D       | EQL-hash picks off this
+  complex-widetag                                 ;  1A   21       | range of widetags.
+  complex-single-float-widetag                    ;  1E   25       |
+  complex-double-float-widetag                    ;  22   29       |
+  ;; -- end of numeric widetags --                                 |
+  symbol-widetag                                  ;  26   2D       /
+  sap-widetag                                     ;  2A   31
 
-  #+64-bit code-header-widetag              ;       31       31
-  #+64-bit instance-widetag                 ;       35       35
-  #+64-bit simple-fun-widetag               ;       39       39
-  #+64-bit funcallable-instance-widetag     ;       3D       3D
-  #+64-bit closure-widetag                  ;       41       41
+  code-header-widetag                             ;  2E   35
+  instance-widetag                                ;  32   39
+  funcallable-instance-widetag                    ;  36   3D
+  simple-fun-widetag                              ;  3A   41
+  closure-widetag                                 ;  3E   45
 
-  ;; x86[-64] does not have objects with this widetag,
-  #-(or x86 x86-64 arm64 riscv) return-pc-widetag ;  3E   45  3E   45
+  #-(or x86 x86-64 arm64 riscv) return-pc-widetag ;  42   49
   #+(or x86 x86-64 arm64 riscv) lra-widetag-notused
 
-  value-cell-widetag                        ;  42   49  42   49
-  character-widetag                         ;  46   4D  46   4D
-  sap-widetag                               ;  4A   51  4A   51
-  #-64-bit unbound-marker-widetag           ;  4E   55  4E   55
+  value-cell-widetag                              ;  46   4D
+  character-widetag                               ;  4A   51
+  #-64-bit unbound-marker-widetag                 ;  4E   55
   #+64-bit unused00-widetag
-  weak-pointer-widetag                      ;  52   59  52   59
-  fdefn-widetag                             ;  56   5D  56   5D
+  weak-pointer-widetag                            ;  52   59
+  fdefn-widetag                                   ;  56   5D
 
-  no-tls-value-marker-widetag               ;  5A   61  5A   61
-  #+sb-simd-pack simd-pack-widetag          ;       65       65
-  #-sb-simd-pack unused01-widetag           ;  5E       5E
-  #+sb-simd-pack-256 simd-pack-256-widetag  ;  62   69  62   69
-  #-sb-simd-pack-256 unused03-widetag       ;  62   69  62   69
-  filler-widetag                            ;  66   6D  66   6D
-  unused04-widetag                          ;  6A   71  6A   71
-  unused05-widetag                          ;  6E   75  6E   75
-  unused06-widetag                          ;  72   79  72   79
-  unused07-widetag                          ;  76   7D  76   7D
-  #-64-bit unused08-widetag                 ;  7A       7A
-  #-64-bit unused09-widetag                 ;  7E       7E
+  unused-widetag                                  ;  5A   61
+  #+sb-simd-pack simd-pack-widetag                ;       65
+  #-sb-simd-pack unused01-widetag                 ;  5E
+  #+sb-simd-pack-256 simd-pack-256-widetag        ;       69
+  #-sb-simd-pack-256 unused03-widetag             ;  62
+  filler-widetag                                  ;  66   6D
+  unused04-widetag                                ;  6A   71
+  unused05-widetag                                ;  6E   75
+  unused06-widetag                                ;  72   79
+  unused07-widetag                                ;  76   7D
+  #-64-bit unused08-widetag                       ;  7A
+  #-64-bit unused09-widetag                       ;  7E
 
-  simple-array-widetag                      ;  82   81  82   81
+  simple-array-widetag                            ;  82   81
   ;; NIL element type is not in the contiguous range of widetags
   ;; corresponding to SIMPLE-UNBOXED-ARRAY
-  simple-array-nil-widetag
+  simple-array-nil-widetag                        ;  86
 
   ;; IF YOU CHANGE THIS ORDER, THEN MANUALLY VERIFY CORRECTNESS OF:
   ;; - leaf_obj_widetag_p()
+  ;; - readonly_unboxed_obj_p()
   ;; - conservative_root_p()
   ;; - anything else I forgot to mention
-  simple-vector-widetag                     ;
-  simple-bit-vector-widetag                 ;
-  simple-array-unsigned-byte-2-widetag      ;
-  simple-array-unsigned-byte-4-widetag      ;
-  simple-array-unsigned-byte-7-widetag      ;
-  simple-array-unsigned-byte-8-widetag      ;
-  simple-array-unsigned-byte-15-widetag     ;
-  simple-array-unsigned-byte-16-widetag     ;
+  simple-vector-widetag                           ;  8A
+  simple-bit-vector-widetag                       ;  8E
+  simple-array-unsigned-byte-2-widetag            ;  92
+  simple-array-unsigned-byte-4-widetag            ;  96
+  simple-array-unsigned-byte-7-widetag            ;  9A
+  simple-array-unsigned-byte-8-widetag            ;  9E
+  simple-array-unsigned-byte-15-widetag           ;  A2
+  simple-array-unsigned-byte-16-widetag           ;  A6
 
-  #-64-bit
-  simple-array-unsigned-fixnum-widetag      ;
-  simple-array-unsigned-byte-31-widetag     ;
-  simple-array-unsigned-byte-32-widetag     ;
-  #+64-bit
-  simple-array-unsigned-fixnum-widetag      ;
-  #+64-bit
-  simple-array-unsigned-byte-63-widetag     ;
-  #+64-bit
-  simple-array-unsigned-byte-64-widetag     ;
-  simple-array-signed-byte-8-widetag        ;
-  simple-array-signed-byte-16-widetag       ;
-  #-64-bit
-  simple-array-fixnum-widetag               ;
-  simple-array-signed-byte-32-widetag       ;
-  #+64-bit
-  simple-array-fixnum-widetag               ;
-  #+64-bit
-  simple-array-signed-byte-64-widetag       ;
-  simple-array-single-float-widetag         ;
-  simple-array-double-float-widetag         ;
-  simple-array-complex-single-float-widetag ;
-  simple-array-complex-double-float-widetag ;
+  #-64-bit simple-array-unsigned-fixnum-widetag   ;
+  simple-array-unsigned-byte-31-widetag           ;
+  simple-array-unsigned-byte-32-widetag           ;
+  #+64-bit simple-array-unsigned-fixnum-widetag   ;
+  #+64-bit simple-array-unsigned-byte-63-widetag  ;
+  #+64-bit simple-array-unsigned-byte-64-widetag  ;
+  simple-array-signed-byte-8-widetag              ;
+  simple-array-signed-byte-16-widetag             ;
+  #-64-bit simple-array-fixnum-widetag            ;
+  simple-array-signed-byte-32-widetag             ;
+  #+64-bit simple-array-fixnum-widetag            ;
+  #+64-bit simple-array-signed-byte-64-widetag    ;
+  simple-array-single-float-widetag               ;
+  simple-array-double-float-widetag               ;
+  simple-array-complex-single-float-widetag       ;
+  simple-array-complex-double-float-widetag       ;
 
   ;; WARNING: If you change the order of anything here,
   ;; be sure to examine COMPUTE-OBJECT-HEADER to see that it works
   ;; properly for all non-simple array headers.
-  simple-base-string-widetag                ;  D6   E1  D6   E1       \
-  #+sb-unicode                              ;                          |
-  simple-character-string-widetag           ;  DA   E5                 | Strings
-  ;; From here down commence the non-simple array types                |
-  complex-base-string-widetag               ;  DE   E9  DA   E5        |
-  #+sb-unicode                              ;                          |
-  complex-character-string-widetag          ;  E2   ED                /
+  simple-base-string-widetag                      ;  D6   E1       \
+  #+sb-unicode simple-character-string-widetag    ;  DA   E5       | Strings
+  #-sb-unicode unused-simple-char-string          ;                |
+  ;; From here down commence the non-simple array types            |
+  complex-base-string-widetag                     ;  DE   E9       |
+  #+sb-unicode complex-character-string-widetag   ;  E2   ED       /
+  #-sb-unicode unused-complex-char-string
 
-  complex-bit-vector-widetag                ;  E6   F1  DE   E9
-  complex-vector-widetag                    ;  EA   F5  E2   ED
-  complex-array-widetag                     ;  EE   F9  E6   F1
-  unused-array-widetag                      ;  F2   FD  EA   F5
+  complex-bit-vector-widetag                      ;  E6   F1
+  complex-vector-widetag                          ;  EA   F5
+  complex-array-widetag                           ;  EE   F9
+  unused-array-widetag                            ;  F2   FD
 ))
 
-;;; Check that INSTANCE and FUNCALLABLE-INSTANCE differ at exactly 1 bit.
+;;; A filler cons whose first word is all 1s looks like this marker pattern,
+;;; but there's no ambiguity, because no-tls-value can't appear in an object
+;;; on the heap.
+(defconstant no-tls-value-marker most-positive-word)
+
+;;; Map each widetag symbol to a string to go in 'tagnames.h'.
+;;; I didn't want to mess with the formatting of the table above.
+(defparameter *widetag-string-alist*
+  `((bignum-widetag "bignum")
+    (ratio-widetag "ratio")
+    (single-float-widetag "sfloat")
+    (double-float-widetag "dfloat")
+    (complex-widetag "cplxnum")
+    (complex-single-float-widetag "cplx-sfloat")
+    (complex-double-float-widetag "cplx-dfloat")
+    (symbol-widetag "symbol")
+    (instance-widetag "instance")
+    (funcallable-instance-widetag "funinstance")
+    (simple-fun-widetag "simplefun")
+    (closure-widetag "closure")
+    (code-header-widetag "codeblob")
+    (return-pc-widetag "LRA")
+    (value-cell-widetag "value-cell")
+    (character-widetag "char")
+    (sap-widetag "sap")
+    (unbound-marker-widetag "unbound-marker")
+    (weak-pointer-widetag "weakptr")
+    (fdefn-widetag "fdefn")
+    (simd-pack-widetag "SIMD-pack")
+    (simd-pack-256-widetag "SIMD-pack256")
+    (filler-widetag "filler")
+    (simple-array-widetag "simple-array")
+    (simple-array-nil-widetag "simple-array-NIL")
+    (simple-vector-widetag "simple-vec")
+    (simple-bit-vector-widetag "simple-bit-vec")
+    (simple-array-fixnum-widetag "fixnum-vec")
+    (simple-array-unsigned-fixnum-widetag "Ufixnum-vec")
+    ;; Autogenerate the arrays. Not all values of N are used, but it's OK
+    ,@(loop for n in '("2" "4" "7" "8" "15" "16" "31" "32" "63" "64")
+            append `((,(symbolicate "SIMPLE-ARRAY-UNSIGNED-BYTE-" n "-WIDETAG")
+                      ,(concatenate 'string "UB" n "-vec"))
+                     (,(symbolicate "SIMPLE-ARRAY-SIGNED-BYTE-" n "-WIDETAG")
+                      ,(concatenate 'string "SB" n "-vec"))))
+    (simple-array-single-float-widetag "sfloat-vec")
+    (simple-array-double-float-widetag "dfloat-vec")
+    (simple-array-complex-single-float-widetag "cplx-sfloat-vec")
+    (simple-array-complex-double-float-widetag "cplx-dfloat-vec")
+    (simple-base-string-widetag "simple-base-str")
+    (simple-character-string-widetag "simple-char-str")
+    ;; I proposed on sbcl-devel some years ago to rename "complex" to "fancy"
+    ;; to avoid confusion with complex numbers. That never gained traction.
+    (complex-base-string-widetag "fancy-base-str")
+    (complex-character-string-widetag "fancy-char-str")
+    (complex-bit-vector-widetag "fancy-bit-vec")
+    (complex-vector-widetag "fancy-vec")
+    (complex-array-widetag "fancy-array")))
+
+(defun widetag-string-name (symbol)
+  ;; Asserting found
+  (the string (second (assoc symbol *widetag-string-alist*))))
+
+;;; Check that INSTANCE and FUNCALLABLE-INSTANCE differ at exactly 1 bit
+;;; and that FUNCALLABLE-INSTANCE is the larger of the two widetags.
 (eval-when (:compile-toplevel)
-  #-64-bit (assert (= (logxor instance-widetag funcallable-instance-widetag)
-                      #b0100))
-  #+64-bit (assert (= (logxor instance-widetag funcallable-instance-widetag)
-                      #b1000)))
+  (assert (= (logcount (logand (logxor instance-widetag funcallable-instance-widetag)
+                               lowtag-mask))
+             1))
+  ;; Note: you must adjust FUNINSTANCE_SELECTOR_BIT_NUMBER (= 2) if the widetags change
+  (assert (= funcallable-instance-widetag
+             (logior instance-widetag (ash 1 2)))))
 
 (defconstant-eqx +function-widetags+
     '#.(list funcallable-instance-widetag simple-fun-widetag closure-widetag)
