@@ -63,10 +63,12 @@
                ;; And of course, don't use them if unsupported.
                ((:fixedobj-space-start fixedobj-space-start*))
                ((:fixedobj-space-size  fixedobj-space-size*) (* 24 1024 1024))
-               ((:varyobj-space-start  varyobj-space-start*))
-               ((:varyobj-space-size   varyobj-space-size*) (* 104 1024 1024))
+               ((:text-space-start text-space-start*))
+               ((:text-space-size  text-space-size*) (* 104 1024 1024))
                (small-space-size #x100000)
-               ((:read-only-space-size ro-space-size) small-space-size))
+               ((:read-only-space-size ro-space-size)
+                #+darwin-jit small-space-size
+                #-darwin-jit 0))
   (declare (ignorable dynamic-space-start*)) ; might be unused in make-host-2
   (flet ((defconstantish (relocatable symbol value)
            (if (not relocatable) ; easy case
@@ -79,23 +81,29 @@
                )))
     (let*
         ((spaces (append `((read-only ,ro-space-size)
+                           #+(and win32 x86-64)
+                           (seh-data ,(symbol-value '+backend-page-bytes+) win64-seh-data-addr)
                            (linkage-table ,small-space-size)
                            #+sb-safepoint
                            ;; Must be just before NIL.
                            (safepoint ,(symbol-value '+backend-page-bytes+) gc-safepoint-page-addr)
-                           (static ,small-space-size))
+                           (static ,small-space-size)
+                           #+darwin-jit
+                           (static-code ,small-space-size))
                          #+immobile-space
                          `((fixedobj ,fixedobj-space-size*)
-                           (varyobj ,varyobj-space-size*))))
+                           (text ,text-space-size*))))
          (ptr small-spaces-start)
          (small-space-forms
            (loop for (space size var-name) in spaces
                  appending
                  (let* ((relocatable
+                          ;; READONLY is usually movable now.
                           ;; TODO: linkage-table could move with code, if the CPU
                           ;; prefers PC-relative jumps, and we emit better code
                           ;; (which we don't- for x86 we jmp via RBX always)
-                          (member space '(fixedobj varyobj)))
+                          (member space '(fixedobj text
+                                          #-darwin-jit read-only)))
                         (start ptr)
                         (end (+ ptr size)))
                    (setf ptr end)
@@ -105,14 +113,14 @@
                          ;; Allow expressly given addresses / sizes for immobile space.
                          ;; The addresses are for testing only - you should not need them.
                          (case space
-                           (varyobj  (setq start (or varyobj-space-start* start)
-                                           end (+ start varyobj-space-size*)))
+                           (text (setq start (or text-space-start* start)
+                                       end (+ start text-space-size*)))
                            (fixedobj (setq start (or fixedobj-space-start* start)
                                            end (+ start fixedobj-space-size*))))
                          `(,(defconstantish relocatable start-sym start)
                            ,(cond ((not relocatable)
                                    `(defconstant ,(symbolicate space "-SPACE-END") ,end))
-                                  #-sb-xc-host ((eq space 'varyobj)) ; don't emit anything
+                                  #-sb-xc-host ((eq space 'text)) ; don't emit anything
                                   (t
                                    `(defconstant ,(symbolicate space "-SPACE-SIZE")
                                       ,(- end start)))))))))))
@@ -127,7 +135,25 @@
            (or ,(or (!read-dynamic-space-size) dynamic-space-size*)
                (ecase n-word-bits
                  (32 (expt 2 29))
-                 (64 (expt 2 30)))))))))
+                 (64 (expt 2 30)))))
+         ;; an arbitrary value to avoid kludging genesis
+         #+(and sb-xc-host (not darwin-jit))
+         (defparameter read-only-space-end read-only-space-start)
+         #-soft-card-marks (defconstant cards-per-page 1)
+         (defconstant gencgc-card-bytes (/ gencgc-page-bytes cards-per-page))
+         (defconstant gencgc-card-shift
+           (integer-length (1- gencgc-card-bytes)))
+         ;; This is a constant during build, but a different value
+         ;; can be patched directly into the affected machine code
+         ;; when the core is loaded based on dynamic-space-size.
+         ;; I think the C runtime does a floor operation rather than ceiling,
+         ;; but firstly there's probably no difference, and secondly it's better
+         ;; to be safe than sorry - using too many bits rather than too few.
+         (defconstant gencgc-card-table-index-nbits
+           (integer-length (1- (ceiling default-dynamic-space-size
+                                        gencgc-card-bytes))))
+         (defconstant gencgc-card-table-index-mask
+           (1- (ash 1 gencgc-card-table-index-nbits)))))))
 
 (defconstant-eqx +c-callable-fdefns+
   '(sub-gc
@@ -145,11 +171,9 @@
     sb-di::handle-breakpoint
     sb-di::handle-single-step-trap
     #+win32 sb-kernel::handle-win32-exception
-    #+sb-thruption sb-thread::run-interruption
+    #+sb-safepoint sb-thread::run-interruption
     enter-alien-callback
-    #+sb-thread sb-thread::enter-foreign-callback
-    #+(and sb-safepoint-strictly (not win32))
-    sb-unix::signal-handler-callback)
+    #+sb-thread sb-thread::enter-foreign-callback)
   #'equal)
 
 ;;; (potentially) static symbols that C code must be able to set/get
@@ -163,16 +187,15 @@
   `((*free-interrupt-context-index* 0)
     (sb-sys:*allow-with-interrupts* t)
     (sb-sys:*interrupts-enabled* t)
-    *alloc-signal*
     sb-sys:*interrupt-pending*
-    #+sb-thruption sb-sys:*thruption-pending*
+    #+sb-safepoint sb-sys:*thruption-pending*
     *in-without-gcing*
     *gc-inhibit*
     *gc-pending*
     #+sb-safepoint sb-impl::*in-safepoint*
     #+sb-thread *stop-for-gc-pending*
-    ;; non-x86oid gencgc object pinning
-    #+(and gencgc (not (or x86 x86-64))) *pinned-objects*
+    *pinned-objects*
+    #+gencgc (*gc-pin-code-pages* 0)
     ;; things needed for non-local-exit
     (*current-catch-block* 0)
     (*current-unwind-protect-block* 0)
@@ -192,11 +215,10 @@
     ,@'(*current-catch-block*
         *current-unwind-protect-block*)
 
-    ;; sb-safepoint in addition to accessing this symbol via TLS,
-    ;; uses the symbol itself as a value. Kinda weird.
-    #+(and sb-safepoint sb-thread) *in-without-gcing*
-
-    #+immobile-space *immobile-freelist* ; not per-thread (yet...)
+    #+metaspace *metaspace-tracts*
+    *immobile-codeblob-tree* ; for generations 0 through 5 inclusive
+    *immobile-codeblob-vector* ; for pseudo-static-generation
+    *dynspace-codeblob-tree*
 
     ;; stack pointers
     #-sb-thread *binding-stack-start* ; a thread slot if #+sb-thread
@@ -208,8 +230,9 @@
     ;; threading support
     #+sb-thread ,@'(sb-thread::*starting-threads* *free-tls-index*)
 
-    ;; dynamic runtime linking support
-    #+linkage-table +required-foreign-symbols+
+    ;; runtime linking of lisp->C calls (regardless of whether
+    ;; the C function is in a dynamic shared object or not)
+    +required-foreign-symbols+
 
     ;;; The following symbols aren't strictly required to be static
     ;;; - they are not accessed from C - but we make them static in order
@@ -228,14 +251,31 @@
 ;;; Refer to the lengthy comment in 'src/runtime/interrupt.h' about
 ;;; the choice of this number. Rather than have to two copies
 ;;; of the comment, please see that file before adjusting this.
-(defconstant max-interrupts 1024)
+;;; I think most of the need for a ridiculously large value stemmed from
+;;; receive-pending-interrupt after a pseudo-atomic code section.
+;;; If that trapped into GC, and then ran finalizers in POST-GC (while still
+;;; in a signal handler), which consed, which caused the need for another GC,
+;;; you'd receive a nested interrupt, as the GC trap was still on the stack
+;;; not having returned to "user" code yet. [See example in src/code/final]
+;;;
+;;; But now that #+sb-thread creates a dedicated finalizer thread, nesting
+;;; seems unlikely to occur, because "your" code doesn't get a chance to run again
+;;; until after the interrupt returns. And the finalizer thread won't invoke
+;;; run-pending-finalizers in post-GC, it will just pick up the next finalizer
+;;; in due turn. You could potentially force a nested GC by consing a lot in
+;;; a post-GC hook, but if you do that, your hook function is badly behaved
+;;; and you should fix it.
+(defconstant max-interrupts
+  #+sb-thread    8  ; reasonable value
+  #-sb-thread 1024) ; crazy value
+
 ;;; Thread slots accessed at negative indices relative to struct thread.
-;;; These slots encroach on the interrupt contexts- the maximum that
-;;; can actually be stored is decreased by this amount.
-;;; sb-safepoint puts the safepoint page immediately preceding the
-;;; thread structure, so this trick doesn't work.
 (defconstant thread-header-slots
-  (+ #+(and x86-64 (not sb-safepoint)) 16))
+  ;; This seems to need to be an even number.
+  ;; I'm not sure what the constraint on that stems from.
+  #+(and x86-64 sb-safepoint) 14 ; the safepoint trap page is at word index -15
+  #+(and x86-64 (not sb-safepoint)) 16
+  #-x86-64 0)
 
 #+gencgc
 (progn
@@ -245,5 +285,47 @@
 (defparameter *runtime-asm-routines* nil)
 (defparameter *linkage-space-predefined-entries* nil)
 
+;;; Floating-point related constants, both format descriptions and FPU
+;;; control register descriptions.  These don't exactly match up with
+;;; what the machine manuals say because the Common Lisp standard
+;;; defines floating-point values somewhat differently than the IEEE
+;;; standard does.
+
+;;; We can currently manipulate only IEEE single and double precision.
+;;; Machine-specific formats (such as x86 80-bit and PPC double-double)
+;;; are unsupported.
+
+#+ieee-floating-point
+(progn
+(defconstant float-sign-shift 31)
+
+;;; The exponent bias is the amount up by which the true exponent is
+;;; incremented for storage purposes.
+;;; (Wikipedia entry for single-float: "an exponent value of 127 represents the actual zero")
+;;; 126 works for us because we actually want an exponent of -1 and not 0
+;;; when using DECODE-FLOAT. -1 is correct because the implied 1 bit in a normalized
+;;; float is the _left_ of of the binary point, so the powers of 2 for the first represented
+;;; bit is 2^-1. Similarly for double-float.
+(defconstant single-float-bias 126)
+(defconstant-eqx single-float-exponent-byte (byte 8 23) #'equalp)
+(defconstant-eqx single-float-significand-byte (byte 23 0) #'equalp)
+(defconstant single-float-normal-exponent-min 1)
+(defconstant single-float-normal-exponent-max 254)
+(defconstant single-float-hidden-bit (ash 1 23))
+
+(defconstant double-float-bias 1022)
+(defconstant-eqx double-float-exponent-byte (byte 11 20) #'equalp)
+(defconstant-eqx double-float-significand-byte (byte 20 0) #'equalp)
+(defconstant double-float-normal-exponent-min 1)
+(defconstant double-float-normal-exponent-max #x7FE)
+(defconstant double-float-hidden-bit (ash 1 20))
+
+(defconstant single-float-digits
+  (+ (byte-size single-float-significand-byte) 1))
+
+(defconstant double-float-digits
+  (+ (byte-size double-float-significand-byte) 32 1))
+)
+
 (push '("SB-VM" +c-callable-fdefns+ +common-static-symbols+)
       *!removable-symbols*)
