@@ -142,6 +142,38 @@
                       (nth n *register-arg-offsets*))
       (make-sc+offset control-stack-sc-number n)))
 
+(defstruct fixed-call-args-state
+  (descriptors -1 :type fixnum)
+  #-c-stack-is-control-stack
+  (non-descriptors -1 :type fixnum)
+  (float -1 :type fixnum))
+
+(declaim (#+sb-xc-host special
+          #-sb-xc-host sb-ext:global
+          *float-regs* *descriptor-args*
+          #-c-stack-is-control-stack *non-descriptor-args*))
+
+(defun fixed-call-arg-location (type state)
+  (let* ((primtype (primitive-type type))
+         (sc (find descriptor-reg-sc-number (sb-c::primitive-type-scs primtype) :test-not #'eql)))
+    (case (primitive-type-name primtype)
+      ((double-float single-float)
+       (make-wired-tn primtype
+                      sc
+                      (elt *float-regs* (incf (fixed-call-args-state-float state)))))
+      ((unsigned-byte-64 signed-byte-64)
+       (make-wired-tn primtype
+                      sc
+                      (elt #-c-stack-is-control-stack *non-descriptor-args*
+                           #+c-stack-is-control-stack *descriptor-args*
+                           (incf (#-c-stack-is-control-stack fixed-call-args-state-non-descriptors
+                                  #+c-stack-is-control-stack fixed-call-args-state-descriptors
+                                  state)))))
+      (t
+       (make-wired-tn primtype
+                      descriptor-reg-sc-number
+                      (elt *descriptor-args* (incf (fixed-call-args-state-descriptors state))))))))
+
 ;;; Make a TN to hold the number-stack frame pointer.  This is allocated
 ;;; once per component, and is component-live.
 (defun make-nfp-tn ()
@@ -241,14 +273,17 @@
   (not (types-equal-or-intersect (tn-ref-type tn-ref)
                                  (specifier-type '(eql nil)))))
 
+(defun instance-tn-ref-p (tn-ref)
+  (csubtypep (tn-ref-type tn-ref) (specifier-type 'instance)))
+
 (defun stack-consed-p (object)
   (let ((write (sb-c::tn-writes object))) ; list of write refs
-    (when (or (not write) ; grrrr, the only write is from a LOAD tn
-                          ; and we don't know the corresponding normal TN?
-              (tn-ref-next write)) ; can't determine if > 1 write
+    (when (or (not write)    ; grrrr, the only write is from a LOAD tn
+                                        ; and we don't know the corresponding normal TN?
+              (tn-ref-next write))      ; can't determine if > 1 write
       (return-from stack-consed-p nil))
     (let ((vop (tn-ref-vop write)))
-      (when (not vop) ; wat?
+      (when (not vop)                   ; wat?
         (return-from stack-consed-p nil))
       (when (eq (vop-name vop) 'allocate-vector-on-stack)
         (return-from stack-consed-p t))
@@ -262,7 +297,7 @@
         (return-from stack-consed-p nil))
       (let* ((splat-input (vop-args vop))
              (splat-input-source
-              (tn-ref-vop (sb-c::tn-writes (tn-ref-tn splat-input)))))
+               (tn-ref-vop (sb-c::tn-writes (tn-ref-tn splat-input)))))
         ;; How in the heck can there NOT be a vop??? Well, sometimes there isn't.
         (when (and splat-input-source
                    (eq (vop-name splat-input-source)
