@@ -22,6 +22,23 @@
 ;;;; specification.
 
 (in-package "SB-PCL")
+
+(defun pcl-compile (expr safety)
+  (let* ((base-policy sb-c::*policy*)
+         (lexenv
+          (sb-c::make-almost-null-lexenv
+           (ecase safety
+             (:safe base-policy)
+             (:unsafe (sb-c::process-optimize-decl
+                       '((space 1) (compilation-speed 1)
+                         (speed 3) (safety 0) (sb-ext:inhibit-warnings 3) (debug 0))
+                       base-policy)))
+           ;; I suspect that INHIBIT-WARNINGS precludes them from happening
+           (list (cons (sb-kernel:find-classoid 'style-warning) 'muffle-warning)
+                 (cons (sb-kernel:find-classoid 'compiler-note) 'muffle-warning))
+           nil nil nil
+           '((:declare sb-c::tlab :system)))))
+    (sb-c:compile-in-lexenv expr lexenv nil nil nil nil nil)))
 
 ;;; GET-FUN is the main user interface to this code. It is like
 ;;; COMPILE, only more efficient. It achieves this efficiency by
@@ -48,19 +65,13 @@
                  (test-converter #'default-test-converter)
                  (code-converter #'default-code-converter)
                  (constant-converter #'default-constant-converter))
-  (function-apply (get-fun-generator lambda test-converter code-converter)
-                  (compute-constants lambda constant-converter)))
-
-(defun get-fun1 (lambda &optional
-                  (test-converter #'default-test-converter)
-                  (code-converter #'default-code-converter)
-                  (constant-converter #'default-constant-converter))
   (values (the function
             (get-fun-generator lambda test-converter code-converter))
           (compute-constants lambda constant-converter)))
 
 (defun default-constantp (form)
-  (constant-typep form '(not (or symbol fixnum cons layout))))
+  (and (constantp form)
+       (not (typep (constant-form-value form) '(or symbol fixnum cons wrapper)))))
 
 (defun default-test-converter (form)
   (if (default-constantp form)
@@ -94,7 +105,7 @@
 
 (defun ensure-fgen (test gensyms generator generator-lambda system)
   (let ((table *fgens*))
-    (sb-thread::with-system-mutex ((hash-table-lock table))
+    (with-system-mutex ((hash-table-lock table))
       (let ((old (gethash test table)))
         (cond (old
                (setf (fgen-generator old) generator)
@@ -108,7 +119,7 @@
 (defun get-fun-generator (lambda test-converter code-converter)
   (let* ((test (compute-test lambda test-converter))
          (table *fgens*)
-         (fgen (sb-thread::with-system-mutex ((hash-table-lock table))
+         (fgen (with-system-mutex ((hash-table-lock table))
                  (gethash test table))))
     (if fgen
         (fgen-generator fgen)
@@ -117,10 +128,9 @@
 (defun get-new-fun-generator (lambda test code-converter)
   (multiple-value-bind (code gensyms) (compute-code lambda code-converter)
     (let ((generator-lambda `(lambda ,gensyms
-                               (declare (muffle-conditions compiler-note)
-                                        (optimize (sb-c:store-source-form 0)))
+                               (declare (optimize (sb-c:store-source-form 0)))
                                (function ,code))))
-      (let ((generator (compile nil generator-lambda)))
+      (let ((generator (pcl-compile generator-lambda :safe)))
         (ensure-fgen test gensyms generator generator-lambda nil)
         generator))))
 
