@@ -63,14 +63,14 @@
 #include "gc.h"
 #include "alloc.h"
 #include "dynbind.h"
-#include "getallocptr.h"
+#include "pseudo-atomic.h"
 #include "genesis/fdefn.h"
 #include "genesis/simple-fun.h"
 #include "genesis/cons.h"
 #include "genesis/vector.h"
+#include "atomiclog.inc"
 
 #ifdef ATOMIC_LOGGING
-#include "atomiclog.inc"
 uword_t *eventdata;
 int n_logevents;
 #endif
@@ -218,7 +218,7 @@ resignal_to_lisp_thread(int signal, os_context_t *context)
         corruption_warning_and_maybe_lose
 #ifdef LISP_FEATURE_SB_THREAD
             ("Received signal %d @ %lx in non-lisp"THREAD_ID_LABEL", resignaling to a lisp thread.",
-             signal, *os_context_pc_addr(context), THREAD_ID_VALUE);
+             signal, os_context_pc(context), THREAD_ID_VALUE);
 #else
             ("Received signal %d in non-lisp thread, resignaling to a lisp thread.", signal);
 #endif
@@ -299,25 +299,49 @@ void dump_eventlog()
 {
     int i = 0;
     uword_t *e = eventdata;
+    char buf[1024];
+    int nc, nc1; // number of chars in buffer
+    // Define buflen to be smaller than 'buf' so that we can prefix it
+    // with thread pointer and suffix it with a newline
+    // without too much hassle.
+#define buflen (sizeof buf-20)
+    nc = snprintf(buf, buflen, "Event log: used %d elements of %d max\n", n_logevents, EVENTBUFMAX);
+    write(2, buf, nc);
     while (i<n_logevents) {
         char *fmt = (char*)e[i+1];
-        switch (e[i]) {
+        uword_t prefix = e[i];
+        int nargs = prefix & 7;
+        void* thread_pointer = (void*)(prefix & ~7);
+        extern char* thread_name_from_pthread(void*);
+        char* name = thread_name_from_pthread(thread_pointer);
+        if (name) nc = sprintf(buf, "%s: ", name); else nc = sprintf(buf, "%p: ", thread_pointer);
+        switch (nargs) {
         default: printf("busted event log"); return;
-        case 2: printf(fmt); break;
-        case 3: printf(fmt,e[i+2]); break;
-        case 4: printf(fmt,e[i+2],e[i+3]); break;
-        case 5: printf(fmt,e[i+2],e[i+3],e[i+4]); break;
-        case 6: printf(fmt,e[i+2],e[i+3],e[i+4],e[i+5]); break;
-        case 7: printf(fmt,e[i+2],e[i+3],e[i+4],e[i+5],e[i+6]); break;
+        case 0: nc1 = snprintf(buf+nc, buflen, fmt, 0); break; // the 0 inhibits a warning
+        case 1: nc1 = snprintf(buf+nc, buflen, fmt, e[i+2]); break;
+        case 2: nc1 = snprintf(buf+nc, buflen, fmt, e[i+2], e[i+3]); break;
+        case 3: nc1 = snprintf(buf+nc, buflen, fmt, e[i+2], e[i+3], e[i+4]); break;
+        case 4: nc1 = snprintf(buf+nc, buflen, fmt, e[i+2], e[i+3], e[i+4], e[i+5]); break;
+        case 5: nc1 = snprintf(buf+nc, buflen, fmt, e[i+2], e[i+3], e[i+4], e[i+5], e[i+6]); break;
+        case 6: nc1 = snprintf(buf+nc, buflen, fmt, e[i+2], e[i+3], e[i+4], e[i+5], e[i+6],
+                               e[i+7]); break;
         }
-        putchar('\n');
-        i += e[i];
+#undef buflen
+        buf[nc+nc1] = '\n';
+        write(2, buf, 1+nc+nc1);
+        i += nargs + 2;
     }
 }
+void sigdump_eventlog(int __attribute__((unused)) signal,
+                      siginfo_t __attribute__((unused)) *info,
+                      os_context_t *context)
+{
+    dump_eventlog();
+}
+
 static void record_signal(int sig, void* context)
 {
-    event3("sig%d @%p in %d", sig, (void*)*os_context_pc_addr(context),
-           (int)get_sb_vm_thread()->os_kernel_tid);
+    event2("got signal %d @ pc=%p", sig, os_context_pc(context));
 }
 #define RECORD_SIGNAL(sig,ctxt) if(sig!=SIGSEGV)record_signal(sig,ctxt);
 #else
@@ -431,6 +455,7 @@ sigaddset_blockable(sigset_t *sigset)
 /* initialized in interrupt_init */
 sigset_t deferrable_sigset;
 sigset_t blockable_sigset;
+sigset_t thread_start_sigset;
 /* gc_sigset will have exactly 1 bit on, for SIG_STOP_FOR_GC, or no bits on.
  * We always use SIGUSR2 as SIG_STOP_FOR_GC, though in days past it may have
  * varied by OS. Also, long ago, there was a different signal to resume after
@@ -523,8 +548,12 @@ check_deferrables_blocked_or_lose(sigset_t *sigset)
         lose("deferrables unblocked");
 }
 
+int sigaction_does_not_mask;
 static void assert_blockables_blocked()
 {
+#ifdef LISP_FEATURE_RISCV
+    if (sigaction_does_not_mask) return; // assert nothing
+#endif
 #if !defined(LISP_FEATURE_WIN32)
     /* On Windows, there are no actual signals, but since the win32 port
      * tracks the sigmask and checks it explicitly, some functions are
@@ -685,7 +714,6 @@ maybe_save_gc_mask_and_block_deferrables(sigset_t *sigset)
 #endif
     if ((!data->pending_handler) &&
         (!data->gc_blocked_deferrables)) {
-        FSHOW_SIGNAL((stderr,"/setting gc_blocked_deferrables\n"));
         data->gc_blocked_deferrables = 1;
         if (sigset) {
             /* This is the sigmask of some context. */
@@ -803,7 +831,24 @@ check_interrupt_context_or_lose(os_context_t *context)
 /*
  * utility routines used by various signal handlers
  */
+#ifdef LISP_FEATURE_ARM64
+static void
+build_fake_control_stack_frames(struct thread __attribute__((unused)) *th,
+                                os_context_t __attribute__((unused)) *context)
+{
 
+    lispobj oldcont;
+    /* Ignore the two words above CSP, which can be used without adjusting CSP */
+    lispobj* csp = (lispobj *)(uword_t) (*os_context_register_addr(context, reg_CSP)) + 2;
+    access_control_frame_pointer(th) = (lispobj *)(uword_t) csp;
+
+    oldcont = (lispobj)(*os_context_register_addr(context, reg_CFP));
+
+    access_control_frame_pointer(th)[1] = os_context_pc(context);
+    access_control_frame_pointer(th)[0] = oldcont;
+    access_control_stack_pointer(th) = csp + 2;
+}
+#else
 static void
 build_fake_control_stack_frames(struct thread __attribute__((unused)) *th,
                                 os_context_t __attribute__((unused)) *context)
@@ -829,7 +874,11 @@ build_fake_control_stack_frames(struct thread __attribute__((unused)) *th,
             access_control_frame_pointer(th)[0] =
                 *os_context_register_addr(context, reg_OCFP);
             access_control_frame_pointer(th)[1] =
-                *os_context_register_addr(context, reg_LRA);
+#ifdef reg_LRA
+              *os_context_register_addr(context, reg_LRA);
+#else
+              *os_context_register_addr(context, reg_RA);
+#endif
             access_control_frame_pointer(th) += 2;
             /* Build our frame on top of it. */
             oldcont = (lispobj)(*os_context_register_addr(context, reg_CFP));
@@ -863,37 +912,23 @@ build_fake_control_stack_frames(struct thread __attribute__((unused)) *th,
     access_control_frame_pointer(th)[2] =
         (lispobj)(*os_context_register_addr(context, reg_CODE));
 #else
-    access_control_frame_pointer(th)[1] = *os_context_pc_addr(context);
+    access_control_frame_pointer(th)[1] = os_context_pc(context);
 #endif
 #endif
 }
+#endif
 
-/* Stores the context for gc to scavange and builds fake stack
+/* Stores the context for gc to scavenge and builds fake stack
  * frames. */
 void fake_foreign_function_call_noassert(os_context_t *context)
 {
     int context_index;
     struct thread *thread=get_sb_vm_thread();
 
-#ifdef reg_ALLOC
-#ifdef LISP_FEATURE_SB_THREAD
-    thread->pseudo_atomic_bits =
-#else
-    dynamic_space_free_pointer =
-        (lispobj *)(uword_t)
-#endif
-            (*os_context_register_addr(context, reg_ALLOC));
-/*     fprintf(stderr,"dynamic_space_free_pointer: %p\n", */
-/*             dynamic_space_free_pointer); */
-#if defined(LISP_FEATURE_MIPS)
-    if ((sword_t)dynamic_space_free_pointer & 1) {
-        lose("dead in fake_foreign_function_call, context = %x", context);
-    }
-#endif
-#endif
 #ifdef reg_BSP
     set_binding_stack_pointer(thread,
-        *os_context_register_addr(context, reg_BSP));
+       // registers can be wider than uword_t (on some 64-bit machines compiling to 32-bit code)
+       (uword_t)*os_context_register_addr(context, reg_BSP));
 #endif
 
 #if defined(LISP_FEATURE_ARM)
@@ -903,8 +938,6 @@ void fake_foreign_function_call_noassert(os_context_t *context)
                   thread);
 #endif
 
-    build_fake_control_stack_frames(thread,context);
-
     /* Do dynamic binding of the active interrupt context index
      * and save the context in the context array. */
     context_index =
@@ -918,10 +951,17 @@ void fake_foreign_function_call_noassert(os_context_t *context)
 
     nth_interrupt_context(context_index, thread) = context;
 
-#if !defined(LISP_FEATURE_X86) && !defined(LISP_FEATURE_X86_64)
+    build_fake_control_stack_frames(thread, context);
+
+#if !defined(LISP_FEATURE_X86) && !defined(LISP_FEATURE_X86_64) &&  \
+  !(defined(LISP_FEATURE_ARM64) && defined(LISP_FEATURE_SB_THREAD))
     /* x86oid targets don't maintain the foreign function call flag at
      * all, so leave them to believe that they are never in foreign
-     * code. */
+     * code.
+
+     And ARM64 uses control_stack_pointer, which is set in
+     build_fake_control_stack_frames. */
+
     foreign_function_call_active_p(thread) = 1;
 #endif
 }
@@ -959,30 +999,6 @@ undo_fake_foreign_function_call(os_context_t __attribute__((unused)) *context)
                    thread);
     unbind(thread);
 #endif
-
-#if defined(reg_ALLOC) && !defined(LISP_FEATURE_SB_THREAD)
-    /* Put the dynamic space free pointer back into the context. */
-    *os_context_register_addr(context, reg_ALLOC) =
-        (uword_t) dynamic_space_free_pointer
-        | (*os_context_register_addr(context, reg_ALLOC)
-           & LOWTAG_MASK);
-    /*
-      ((uword_t)(*os_context_register_addr(context, reg_ALLOC))
-      & ~LOWTAG_MASK)
-      | ((uword_t) dynamic_space_free_pointer & LOWTAG_MASK);
-    */
-#endif
-#if defined(reg_ALLOC) && defined(LISP_FEATURE_SB_THREAD)
-    /* Put the pseudo-atomic bits and dynamic space free pointer back
-     * into the context (p-a-bits for p-a, and dynamic space free
-     * pointer for ROOM). */
-    *os_context_register_addr(context, reg_ALLOC) =
-        (uword_t) dynamic_space_free_pointer
-        | (thread->pseudo_atomic_bits & LOWTAG_MASK);
-    /* And clear them so we don't get bit later by call-in/call-out
-     * not updating them. */
-    thread->pseudo_atomic_bits = 0;
-#endif
 }
 
 /* a handler for the signal caused by execution of a trap opcode
@@ -1013,13 +1029,11 @@ interrupt_internal_error(os_context_t *context, boolean continuable)
     }
 #endif
 
-    SHOW("in interrupt_internal_error");
-#if QSHOW == 2
     /* Display some rudimentary debugging information about the
      * error, so that even if the Lisp error handler gets badly
      * confused, we have a chance to determine what's going on. */
-    describe_internal_error(context);
-#endif
+    // describe_internal_error(context); // uncomment me for debugging
+
     funcall2(StaticSymbolFunction(INTERNAL_ERROR), context_sap,
              continuable ? T : NIL);
 
@@ -1066,8 +1080,6 @@ interrupt_handle_pending(os_context_t *context)
         lose("Handling pending interrupt in pseudo atomic.");
     }
 
-    FSHOW_SIGNAL((stderr, "/entering interrupt_handle_pending\n"));
-
     assert_blockables_blocked();
 #ifndef LISP_FEATURE_SB_SAFEPOINT
     /*
@@ -1213,9 +1225,8 @@ interrupt_handle_pending(os_context_t *context)
         lose("pseudo_atomic_interrupted after interrupt_handle_pending");
 #endif
     /* It is possible that the end of this function was reached
-     * without never actually doing anything, the tests in Lisp for
+     * without actually doing anything, the tests in Lisp for
      * when to call receive-pending-interrupt are not exact. */
-    FSHOW_SIGNAL((stderr, "/exiting interrupt_handle_pending\n"));
 }
 
 
@@ -1258,8 +1269,6 @@ interrupt_handle_now(int signal, siginfo_t *info, os_context_t *context)
             DX_ALLOC_SAP(context_sap, context);
             DX_ALLOC_SAP(info_sap, info);
 
-            FSHOW_SIGNAL((stderr,"/calling Lisp-level handler\n"));
-
             funcall3(handler,
                      make_fixnum(signal),
                      info_sap,
@@ -1288,7 +1297,6 @@ run_deferred_handler(struct interrupt_data *data, os_context_t *context)
         data->pending_handler;
 
     data->pending_handler=0;
-    FSHOW_SIGNAL((stderr, "/running deferred handler %p\n", pending_handler));
     (*pending_handler)(data->pending_signal,&(data->pending_info), context);
 }
 
@@ -1316,6 +1324,25 @@ store_signal_data_for_later (struct interrupt_data *data, void *handler,
     sigaddset_deferrable(os_context_sigmask_addr(context));
 }
 
+/* What's going on ?
+ *
+ *  0: fp=0x7fa86423e7a0 pc=0x27608c Foreign function (null)
+ *  1: fp=0x7fa86423e7b0 pc=0x2767aa Foreign function (null)
+ *  2: fp=0x7fa86423e890 pc=0x2761af Foreign function (null)
+ *  3: fp=0x7fa86423e970 pc=0x27882f Foreign function (null)       ; maybe_now_maybe_later
+ *  4: fp=0x7fa86423f5c0 pc=0x7fa8646bc750 Foreign function (null) ; WHICH SIGNAL ?
+ *  5: fp=0x7fa86423f630 pc=0x2713ce Foreign function (null)       ; verify_range
+ *  6: fp=0x7fa86423f6c0 pc=0x2703d3 Foreign function (null)       ; verify_heap
+ *  7: fp=0x7fa86423f780 pc=0x26e168 Foreign function collect_garbage
+ *  8: fp=0x7fa86423f7f0 pc=0x270298 Foreign function gc_and_save
+ *  9: fp=0x7fa86423f848 pc=0x52f93bf2 <??? type 45>::GC-AND-SAVE
+ * 10: fp=0x7fa86423f950 pc=0x52d32a7b <??? type 45>::SAVE-LISP-AND-DIE
+ * 11: fp=0x7fa86423fa00 pc=0x52a34179 <??? type 45>::SAVE-LISP-AND-DIE
+ *
+ * fatal error encountered in SBCL pid 9436 tid 9436:
+ * interrupt already pending
+ */
+
 static boolean
 can_handle_now(void *handler, struct interrupt_data *data,
                int signal, siginfo_t *info, os_context_t *context)
@@ -1329,7 +1356,8 @@ can_handle_now(void *handler, struct interrupt_data *data,
     struct thread *thread = get_sb_vm_thread();
 
     if (read_TLS(INTERRUPT_PENDING,thread) != NIL)
-        lose("interrupt already pending");
+        lose("interrupt already pending when sig%d received, pc=%p", signal,
+             (void*)os_context_pc(context));
     if (thread_interrupt_data(thread).pending_handler)
         lose("there is a pending handler already (PA)");
     if (data->gc_blocked_deferrables)
@@ -1346,10 +1374,8 @@ can_handle_now(void *handler, struct interrupt_data *data,
      */
     if ((read_TLS(INTERRUPTS_ENABLED,thread) == NIL) ||
         in_leaving_without_gcing_race_p(thread)) {
-        FSHOW_SIGNAL((stderr,
-                      "/can_handle_now(%p,%d): deferred (RACE=%d)\n",
-                      handler,signal,
-                      in_leaving_without_gcing_race_p(thread)));
+        event3("can_handle_now(%p,%d): deferred (RACE=%d)", handler, signal,
+               in_leaving_without_gcing_race_p(thread));
         store_signal_data_for_later(data,handler,signal,info,context);
         write_TLS(INTERRUPT_PENDING, T,thread);
         answer = 0;
@@ -1358,9 +1384,7 @@ can_handle_now(void *handler, struct interrupt_data *data,
      * actually use its argument for anything on x86, so this branch
      * may succeed even when context is null (gencgc alloc()) */
     else if (arch_pseudo_atomic_atomic(context)) {
-        FSHOW_SIGNAL((stderr,
-                      "/can_handle_now(%p,%d): deferred(PA)\n",
-                      handler,signal));
+        event2("can_handle_now(%p,%d): deferred (PA)", handler, signal);
         store_signal_data_for_later(data,handler,signal,info,context);
         arch_set_pseudo_atomic_interrupted(context);
         answer = 0;
@@ -1402,11 +1426,11 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
     /* Test for GC_INHIBIT _first_, else we'd trap on every single
      * pseudo atomic until gc is finally allowed. */
     if (read_TLS(GC_INHIBIT,thread) != NIL) {
-        FSHOW_SIGNAL((stderr, "sig_stop_for_gc deferred (*GC-INHIBIT*)\n"));
+        event0("stop_for_gc deferred for *GC-INHIBIT*");
         write_TLS(STOP_FOR_GC_PENDING,T,thread);
         return;
     } else if (arch_pseudo_atomic_atomic(context)) {
-        FSHOW_SIGNAL((stderr,"sig_stop_for_gc deferred (PA)\n"));
+        event0("stop_for_gc deferred for PA");
         write_TLS(STOP_FOR_GC_PENDING,T,thread);
         arch_set_pseudo_atomic_interrupted(context);
         maybe_save_gc_mask_and_block_deferrables
@@ -1414,7 +1438,7 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
         return;
     }
 
-    FSHOW_SIGNAL((stderr, "/sig_stop_for_gc_handler\n"));
+    event0("stop_for_gc");
 
     /* Not PA and GC not inhibited -- we can stop now. */
 
@@ -1439,7 +1463,7 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
      * GC. GC_BLOCKED_DEFERRABLES is also left at 1. So let's tidy it
      * up. */
     if (thread_interrupt_data(thread).gc_blocked_deferrables) {
-        FSHOW_SIGNAL((stderr,"cleaning up after gc_blocked_deferrables\n"));
+        event0("cleaning up after gc_blocked_deferrables");
         clear_pseudo_atomic_interrupted(thread);
         struct interrupt_data *interrupt_data = &thread_interrupt_data(thread);
         sigcopyset(os_context_sigmask_addr(context), &interrupt_data->pending_mask);
@@ -1457,7 +1481,7 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
      * occurs below at thread_wait_until_not(STATE_STOPPED). Note that sem_post()
      * is expressly permitted in signal handlers, and set_thread_state uses it */
     set_thread_state(thread, STATE_STOPPED, 0);
-    FSHOW_SIGNAL((stderr,"suspended\n"));
+    event0("suspended");
 
     /* While waiting for gc to finish occupy ourselves with zeroing
      * the unused portion of the control stack to reduce conservatism.
@@ -1492,7 +1516,7 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
     int my_state = thread_wait_until_not(STATE_STOPPED, thread);
 #endif
 
-    FSHOW_SIGNAL((stderr,"resumed\n"));
+    event0("resumed");
 
     /* The state can't go from STOPPED to DEAD because it's this thread is reading
      * its own state, hence it must be running.
@@ -1519,7 +1543,7 @@ interrupt_handle_now_handler(int signal, siginfo_t *info, void *void_context)
 #endif
         )
         corruption_warning_and_maybe_lose("Signal %d received (PC: %p)", signal,
-                                          *os_context_pc_addr(context));
+                                          os_context_pc(context));
 #endif
     interrupt_handle_now(signal, info, context);
     RESTORE_ERRNO;
@@ -1531,7 +1555,7 @@ interrupt_handle_now_handler(int signal, siginfo_t *info, void *void_context)
  */
 
 #if (defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64))
-extern int *context_eflags_addr(os_context_t *context);
+extern int *os_context_flags_addr(os_context_t *context);
 #endif
 
 extern lispobj call_into_lisp(lispobj fun, lispobj *args, int nargs);
@@ -1599,10 +1623,6 @@ arrange_return_to_c_function(os_context_t *context,
 #if defined(LISP_FEATURE_DARWIN)
     uint32_t *register_save_area = (uint32_t *)os_allocate(0x40);
 
-    FSHOW_SIGNAL((stderr, "/arrange_return_to_lisp_function: preparing to go to function %x, sp: %x\n", function,
-                  *os_context_register_addr(context,reg_ESP)));
-    FSHOW_SIGNAL((stderr, "/arrange_return_to_lisp_function: context: %x, &context %x\n", context, &context));
-
     /* 1. os_validate (malloc/mmap) register_save_block
      * 2. copy register state into register_save_block
      * 3. put a pointer to register_save_block in a register in the context
@@ -1612,7 +1632,7 @@ arrange_return_to_c_function(os_context_t *context,
      *    c. calls the function
      */
 
-    *register_save_area = *os_context_pc_addr(context);
+    *register_save_area = os_context_pc(context);
     *(register_save_area + 1) = function;
     *(register_save_area + 2) = *os_context_register_addr(context,reg_EDI);
     *(register_save_area + 3) = *os_context_register_addr(context,reg_ESI);
@@ -1620,10 +1640,9 @@ arrange_return_to_c_function(os_context_t *context,
     *(register_save_area + 5) = *os_context_register_addr(context,reg_ECX);
     *(register_save_area + 6) = *os_context_register_addr(context,reg_EBX);
     *(register_save_area + 7) = *os_context_register_addr(context,reg_EAX);
-    *(register_save_area + 8) = *context_eflags_addr(context);
+    *(register_save_area + 8) = *os_context_flags_addr(context);
 
-    *os_context_pc_addr(context) =
-      (os_context_register_t) funptr;
+    set_os_context_pc(context, (os_context_register_t) funptr);
     *os_context_register_addr(context,reg_ECX) =
       (os_context_register_t) register_save_area;
 #else
@@ -1645,9 +1664,9 @@ arrange_return_to_c_function(os_context_t *context,
     *(sp-6)=*os_context_register_addr(context,reg_EDX);
     *(sp-5)=*os_context_register_addr(context,reg_ECX);
     *(sp-4)=*os_context_register_addr(context,reg_EAX);
-    *(sp-3)=*context_eflags_addr(context);
+    *(sp-3)=*os_context_flags_addr(context);
     *(sp-2)=*os_context_register_addr(context,reg_EBP);
-    *(sp-1)=*os_context_pc_addr(context);
+    *(sp-1)=os_context_pc(context);
 
 #endif
 
@@ -1672,9 +1691,9 @@ arrange_return_to_c_function(os_context_t *context,
     *(sp-6)=*os_context_register_addr(context,reg_RDX);
     *(sp-5)=*os_context_register_addr(context,reg_RCX);
     *(sp-4)=*os_context_register_addr(context,reg_RAX);
-    *(sp-3)=*context_eflags_addr(context);
+    *(sp-3)=*os_context_flags_addr(context);
     *(sp-2)=*os_context_register_addr(context,reg_RBP);
-    *(sp-1)=*os_context_pc_addr(context);
+    *(sp-1)=os_context_pc(context);
 
     *os_context_register_addr(context,reg_RDI) =
         (os_context_register_t)function; /* function */
@@ -1688,7 +1707,7 @@ arrange_return_to_c_function(os_context_t *context,
 #ifdef LISP_FEATURE_X86
 
 #if !defined(LISP_FEATURE_DARWIN)
-    *os_context_pc_addr(context) = (os_context_register_t)funptr;
+    set_os_context_pc(context, (os_context_register_t)funptr);
     *os_context_register_addr(context,reg_ECX) = 0;
     *os_context_register_addr(context,reg_EBP) = (os_context_register_t)(sp-2);
 #ifdef __NetBSD__
@@ -1700,14 +1719,14 @@ arrange_return_to_c_function(os_context_t *context,
 #endif /* LISP_FEATURE_DARWIN */
 
 #elif defined(LISP_FEATURE_X86_64)
-    *os_context_pc_addr(context) = (os_context_register_t)funptr;
+    set_os_context_pc(context, (os_context_register_t)funptr);
     *os_context_register_addr(context,reg_RCX) = 0;
     *os_context_register_addr(context,reg_RBP) = (os_context_register_t)(sp-2);
     *os_context_register_addr(context,reg_RSP) = (os_context_register_t)(sp-18);
 #else
     /* this much of the calling convention is common to all
        non-x86 ports */
-    *os_context_pc_addr(context) = (os_context_register_t)(unsigned long)code;
+    set_os_context_pc(context, (os_context_register_t)(unsigned long)code);
     *os_context_register_addr(context,reg_NARGS) = 0;
 #ifdef reg_LIP
     *os_context_register_addr(context,reg_LIP) =
@@ -1717,8 +1736,7 @@ arrange_return_to_c_function(os_context_t *context,
         (os_context_register_t)(unsigned long)access_control_frame_pointer(th);
 #endif
 #ifdef ARCH_HAS_NPC_REGISTER
-    *os_context_npc_addr(context) =
-        4 + *os_context_pc_addr(context);
+    *os_context_npc_addr(context) = 4 + os_context_pc(context);
 #endif
 #if defined(LISP_FEATURE_SPARC) || defined(LISP_FEATURE_ARM) || defined(LISP_FEATURE_RISCV)
     *os_context_register_addr(context,reg_CODE) =
@@ -1781,7 +1799,7 @@ handle_guard_page_triggered(os_context_t *context,os_vm_address_t addr)
        addr < CONTROL_STACK_HARD_GUARD_PAGE(th) + os_vm_page_size) {
         fake_foreign_function_call(context);
         lose("Control stack exhausted, fault: %p, PC: %p",
-             addr, (void*)*os_context_pc_addr(context));
+             addr, (void*)os_context_pc(context));
     }
     else if(addr >= CONTROL_STACK_GUARD_PAGE(th) &&
             addr < CONTROL_STACK_GUARD_PAGE(th) + os_vm_page_size) {
@@ -1792,17 +1810,17 @@ handle_guard_page_triggered(os_context_t *context,os_vm_address_t addr)
         if (gc_active_p) {
             fake_foreign_function_call(context);
             lose("Control stack exhausted with gc_active_p, fault: %p, PC: %p",
-                 addr, (void*)*os_context_pc_addr(context));
+                 addr, (void*)os_context_pc(context));
         }
         if (arch_pseudo_atomic_atomic(context)) {
             fake_foreign_function_call(context);
             lose("Control stack exhausted while pseudo-atomic, fault: %p, PC: %p",
-                 addr, (void*)*os_context_pc_addr(context));
+                 addr, (void*)os_context_pc(context));
         }
         if (lose_on_corruption_p) {
             fake_foreign_function_call(context);
             lose("Control stack exhausted, fault: %p, PC: %p",
-                 addr, (void*)*os_context_pc_addr(context));
+                 addr, (void*)os_context_pc(context));
         }
         if (!th->state_word.control_stack_guard_page_protected)
             lose("control_stack_guard_page_protected NIL");
@@ -2017,10 +2035,15 @@ void
 interrupt_init(void)
 {
 #ifdef ATOMIC_LOGGING
-    eventdata = calloc(EVENTBUFMAX, N_WORD_BYTES);
+    // If fetch_and_add gives us an index that is less than EVENTBUFMAX,
+    // we assume that there is room to record an event with up to 8 arguments
+    // which means the prefix, the format string, and the arguments.
+    eventdata = calloc(EVENTBUFMAX+10, N_WORD_BYTES);
+    void sigdump_eventlog(int, siginfo_t*, os_context_t*);
+    // pick anything not used. SIGPWR is also a good choice
+    ll_install_handler(SIGINFO, sigdump_eventlog);
 #endif
     int __attribute__((unused)) i;
-    SHOW("entering interrupt_init()");
     sigemptyset(&deferrable_sigset);
     sigemptyset(&blockable_sigset);
     sigemptyset(&gc_sigset);
@@ -2028,6 +2051,14 @@ interrupt_init(void)
     sigaddset_blockable(&blockable_sigset);
     sigaddset_gc(&gc_sigset);
 
+    sigaddset_deferrable(&thread_start_sigset);
+    /* sigprof_handler may interrupt a thread that doesn't have
+     current_thread set up yet, which can be a thread-local variable,
+     and sigprof_handler will try to allocate it, but thread-local
+     initialization is not guaranteed to be async safe. */
+    sigaddset(&thread_start_sigset, SIGPROF);
+
+
 #ifdef LISP_FEATURE_BACKTRACE_ON_SIGNAL
     // Use this only if you know what you're doing
     void backtrace_lisp_threads(int, siginfo_t*, os_context_t*);
@@ -2052,9 +2083,13 @@ lisp_memory_fault_error(os_context_t *context, os_vm_address_t addr)
     /* If we lose on corruption, provide LDB with debugging information. */
     fake_foreign_function_call(context);
 
+    /* If it's a store to read-only space, it's not "corruption", so don't say that.
+     * Lisp will change its wording of the memory-fault-error string */
+
+    if (!readonly_space_p((uword_t)addr)) {
     /* To allow debugging memory faults in signal handlers and such. */
 #ifdef ARCH_HAS_STACK_POINTER
-    char* pc = (char*)*os_context_pc_addr(context);
+    char* pc = (char*)os_context_pc(context);
     struct code* code = (struct code*)component_ptr_from_pc(pc);
     unsigned int offset = code ? pc - (char*)code : 0;
     if (offset)
@@ -2070,8 +2105,9 @@ lisp_memory_fault_error(os_context_t *context, os_vm_address_t addr)
             *os_context_sp_addr(context), THREAD_ID_VALUE);
 #else
     corruption_warning_and_maybe_lose("Memory fault at %p (pc=%p)",
-                                      addr, *os_context_pc_addr(context));
+                                      addr, (void*)os_context_pc(context));
 #endif
+    }
 
 #ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
     /* Holy hell is this more obfuscated than necessary when using
@@ -2100,7 +2136,7 @@ lisp_memory_fault_error(os_context_t *context, os_vm_address_t addr)
     extern void memory_fault_emulation_trap(void);
     undo_fake_foreign_function_call(context);
     void **sp = (void **)*os_context_sp_addr(context);
-    *--sp = (void *)*os_context_pc_addr(context);
+    *--sp = (void *)os_context_pc(context);
     *--sp = addr;
 #  ifdef LISP_FEATURE_X86
     /* KLUDGE: x86-linux sp_addr doesn't affect the CPU on return */
@@ -2108,8 +2144,7 @@ lisp_memory_fault_error(os_context_t *context, os_vm_address_t addr)
 #  else
     *((void **)os_context_sp_addr(context)) = sp;
 #  endif
-    *os_context_pc_addr(context) =
-        (os_context_register_t)memory_fault_emulation_trap;
+    set_os_context_pc(context, (os_context_register_t)memory_fault_emulation_trap);
     /* We exit here, letting the signal handler return, picking up at
      * memory_fault_emulation_trap (in target-assem.S), which will
      * trap, and the handler calls the function below, where we
@@ -2122,7 +2157,7 @@ handle_memory_fault_emulation_trap(os_context_t *context)
 {
     void **sp = (void **)*os_context_sp_addr(context);
     void *addr = *sp++;
-    *os_context_pc_addr(context) = (os_context_register_t)*sp++;
+    set_os_context_pc(context, (os_context_register_t)*sp++);
 #  ifdef LISP_FEATURE_X86
     /* KLUDGE: x86-linux sp_addr doesn't affect the CPU on return */
     *((void **)os_context_register_addr(context, reg_ESP)) = sp;
